from typing import Any, Dict, List
from unittest import TestCase
from unittest.mock import Mock, call, patch

import click

from montecarlodata.utils import AwsClientWrapper

_SAMPLE_FILE_PATH = "file"
_SAMPLE_BUCKET_NAME = "bucket"
_SAMPLE_OBJECT_NAME = "obj"
_SAMPLE_REGION = "bar"
_SAMPLE_PROFILE_NAME = "foo"
_SAMPLE_STACK_ID = "arn:aws:stack"
_SAMPLE_STACK_LINK = "link"
_SAMPLE_STACK_PARAMS = [{"foo": "bar"}]
_SAMPLE_STACK_CAPABILITIES = "CAPABILITY_IAM"
_SAMPLE_STACK_WAIT_CONFIG = {"Delay": 5, "MaxAttempts": 720}
_SAMPLE_GATEWAY_ID = "id"
_SAMPLE_GATEWAY_STAGE = "stage"
_SAMPLE_STACK = {"Stacks": [{"Outputs": [0, 1, 2]}]}
_SAMPLE_TERM_PROTECT = False


class MockClient:
    # Arguments, classes and functions use AWS casing...

    @staticmethod
    def describe_stacks(StackName: str) -> Any:
        assert StackName == _SAMPLE_STACK_ID
        return _SAMPLE_STACK

    @staticmethod
    def upload_file(file_path: str, bucket_name: str, object_name: str):
        assert _SAMPLE_FILE_PATH == file_path
        assert _SAMPLE_BUCKET_NAME == bucket_name
        assert _SAMPLE_OBJECT_NAME == object_name

    @staticmethod
    def update_stack(
        StackName: str,
        TemplateURL: str,
        Parameters: List[Dict],
        Capabilities: List[str],
    ):
        assert StackName == _SAMPLE_STACK_ID
        assert TemplateURL == _SAMPLE_STACK_LINK
        assert Parameters == _SAMPLE_STACK_PARAMS
        assert Capabilities == [_SAMPLE_STACK_CAPABILITIES]

    @staticmethod
    def create_stack(
        StackName: str, TemplateURL: str, Parameters: List[Dict], Capabilities: List[str], **kwargs
    ):
        assert StackName == _SAMPLE_STACK_ID
        assert TemplateURL == _SAMPLE_STACK_LINK
        assert Parameters == _SAMPLE_STACK_PARAMS[0]
        assert Capabilities == [_SAMPLE_STACK_CAPABILITIES]
        return {"StackId": StackName}

    @staticmethod
    def create_deployment(restApiId: str, stageName: str):
        assert restApiId == _SAMPLE_GATEWAY_ID
        assert stageName == _SAMPLE_GATEWAY_STAGE

    class get_waiter:
        def __init__(self, waiter):
            self.wait = self._wait

        def _wait(self, StackName, WaiterConfig):
            assert StackName == _SAMPLE_STACK_ID
            assert WaiterConfig == _SAMPLE_STACK_WAIT_CONFIG


class AwsUtilTest(TestCase):
    def setUp(self) -> None:
        self._session_mock = Mock()

        self._service = self._setup()
        self._session_mock.reset_mock()

        self._session_mock.client.return_value = MockClient

    def test_setup(self):
        self._setup()

    def test_get_stack_outputs(self):
        self._service.get_stack_outputs(_SAMPLE_STACK_ID)
        self._session_mock.client.assert_called_once_with("cloudformation")

    def test_get_stack_parameters(self):
        self._service.get_stack_parameters(_SAMPLE_STACK_ID)
        self._session_mock.client.assert_called_once_with("cloudformation")

    @patch.object(AwsClientWrapper, "get_stack_details")
    def test_upgrade_stack(self, mock_get_stack_details):
        mock_get_stack_details.return_value = {"Stacks": [{"StackStatus": "UPDATE_COMPLETE"}]}
        self.assertTrue(
            self._service.upgrade_stack(_SAMPLE_STACK_ID, _SAMPLE_STACK_LINK, _SAMPLE_STACK_PARAMS)
        )
        self._session_mock.client.assert_has_calls = [call("cloudformation") * 2]

    @patch.object(AwsClientWrapper, "get_stack_details")
    def test_upgrade_stack_failed(self, mock_get_stack_details):
        mock_get_stack_details.return_value = {"Stacks": [{"StackStatus": "UPDATE_FAILED"}]}
        self.assertFalse(
            self._service.upgrade_stack(_SAMPLE_STACK_ID, _SAMPLE_STACK_LINK, _SAMPLE_STACK_PARAMS)
        )
        self._session_mock.client.assert_has_calls = [call("cloudformation") * 2]

    @patch.object(AwsClientWrapper, "get_stack_details")
    def test_create_stack(self, mock_get_stack_details):
        mock_get_stack_details.return_value = {"Stacks": [{"StackStatus": "CREATE_COMPLETE"}]}
        self.assertTrue(
            self._service.create_stack(
                _SAMPLE_STACK_ID, _SAMPLE_STACK_LINK, False, _SAMPLE_STACK_PARAMS[0]
            )
        )
        self._session_mock.client.assert_has_calls = [call("cloudformation") * 2]

    @patch.object(AwsClientWrapper, "get_stack_details")
    def test_create_stack_failed(self, mock_get_stack_details):
        mock_get_stack_details.return_value = {"Stacks": [{"StackStatus": "CREATE_FAILED"}]}
        self.assertFalse(
            self._service.create_stack(
                _SAMPLE_STACK_ID, _SAMPLE_STACK_LINK, False, _SAMPLE_STACK_PARAMS[0]
            )
        )
        self._session_mock.client.assert_has_calls = [call("cloudformation") * 2]

    def test_deploy_gateway(self):
        self._service.deploy_gateway(_SAMPLE_GATEWAY_ID, _SAMPLE_GATEWAY_STAGE)
        self._session_mock.client.assert_called_once_with("apigateway")

    def test_upload_file(self):
        self._service.upload_file(
            file_path=_SAMPLE_FILE_PATH,
            bucket_name=_SAMPLE_BUCKET_NAME,
            object_name=_SAMPLE_OBJECT_NAME,
        )
        self._session_mock.client.assert_called_once_with("s3")

    def test_region_error(self):
        self._session_mock.client.side_effect = ValueError()
        with self.assertRaises(click.exceptions.Abort):
            self._setup()

    def _setup(self) -> AwsClientWrapper:
        client = AwsClientWrapper(_SAMPLE_PROFILE_NAME, _SAMPLE_REGION, session=self._session_mock)

        self._session_mock.client.assert_called_once_with("cloudformation")
        return client



from typing import Optional

import click

from montecarlodata.common import create_mc_client
from montecarlodata.secrets.service import AccountSecretsService


@click.group(help="Manage account secrets.")
def secrets():
    pass


@secrets.command(name="create", help="Add a new secret to the account.")
@click.pass_obj
@click.option(
    "--name",
    required=True,
    help="Name of the secret, to reference it when using the secret.",
)
@click.option(
    "--scope",
    required=False,
    help="Scope where the secret can be used.",
    default="global",
    show_default=True,
)
@click.option(
    "--value",
    prompt="Secret value",
    required=True,
    hide_input=True,
    help="Secret value",
)
@click.option(
    "--expires-at",
    required=False,
    type=click.DateTime(formats=["%Y-%m-%d"]),
    help="A date in the future when the secret should expire and no longer be readable.",
)
@click.option("--description", required=False, help="Description for the secret.")
def create_secret(
    ctx,
    name: str,
    scope: str,
    value: str,
    description: Optional[str] = None,
    expires_at: Optional[click.DateTime] = None,
):
    AccountSecretsService(
        config=ctx["config"],
        pycarlo_client=create_mc_client(ctx),
        command_name="secrets create_secret",
    ).create_secret(
        name,
        scope=scope,
        value=value,
        description=description,
        expires_at=expires_at,  # type: ignore
    )


@secrets.command(
    name="get", help="Get a secret properties, optionally requesting the secret value."
)
@click.pass_obj
@click.option(
    "--name",
    required=True,
    help="Name of the secret.",
)
@click.option(
    "--reveal",
    required=False,
    help="Show the secret value. Only the owner of the secret or user with special permissions can "
    "get the secret value.",
    default=False,
    show_default=True,
    is_flag=True,
)
def get_secret(ctx, name: str, reveal: bool):
    AccountSecretsService(
        config=ctx["config"],
        pycarlo_client=create_mc_client(ctx),
        command_name="secrets get_secret",
    ).get_secret(name, reveal=reveal)


@secrets.command(name="list", help="List all the secrets in the account.")
@click.pass_obj
def list_secrets(ctx):
    AccountSecretsService(
        config=ctx["config"],
        pycarlo_client=create_mc_client(ctx),
        command_name="secrets list_secrets",
    ).list_secrets()


@secrets.command(name="delete", help="Delete a secret from the account.")
@click.pass_obj
@click.option(
    "--name",
    required=True,
    help="Name of the secret.",
)
def delete_secret(ctx, name: str):
    AccountSecretsService(
        config=ctx["config"],
        pycarlo_client=create_mc_client(ctx),
        command_name="secrets delete_secret",
    ).delete_secret(name)



import uuid
from unittest import TestCase
from unittest.mock import Mock, patch

import click
from box import Box, BoxList

from montecarlodata.common.data import MonolithResponse
from montecarlodata.common.user import UserService
from montecarlodata.config import Config
from montecarlodata.integrations.keys import (
    IntegrationKeyService,
    Queries,
)
from montecarlodata.utils import GqlWrapper


class IntegrationKeyServiceTests(TestCase):
    config = Config(
        mcd_id="12345",
        mcd_token="67890",
        mcd_api_endpoint="https://api.getmontecarlo.com/graphql",
    )

    def setUp(self) -> None:
        self._request_wrapper = Mock(autospec=GqlWrapper)
        self._user_service = Mock(autospec=UserService)
        self._service = IntegrationKeyService(
            config=self.config,
            command_name="test",
            gql=self._request_wrapper,
            user_service=self._user_service,
        )

    @patch.object(click, "echo")
    def test_create_spark_key(self, echo: Mock):
        # given
        description = "test key"
        scope = "spark"
        warehouses = [
            {"uuid": "lake", "connectionType": "DATA_LAKE"},
            {"uuid": "snowflake", "connectionType": "SNOWFLAKE"},
        ]
        key = {"id": "key-id", "secret": "key-secret"}

        self._user_service.warehouses = warehouses
        self._request_wrapper.make_request_v2.return_value = MonolithResponse(
            data=Box({"key": key}), errors=None
        )

        # when
        self._service.create(description=description, scope=scope)

        # then
        self._request_wrapper.make_request_v2.assert_called_once_with(
            query=Queries.create.query,
            operation=Queries.create.operation,
            service="integration_key_service",
            variables={
                "description": description,
                "scope": scope,
                "warehouseIds": [warehouses[0]["uuid"]],
            },
        )

        echo_calls = echo.call_args_list
        self.assertEqual(2, len(echo_calls))
        self.assertEqual(f"Key id: {key['id']}", echo_calls[0][0][0])
        self.assertEqual(f"Key secret: {key['secret']}", echo_calls[1][0][0])

    @patch.object(click, "echo")
    def test_create_spark_key_with_multiple_lakes(self, echo: Mock):
        # given
        description = "test key"
        scope = "spark"
        warehouses = [
            {"uuid": "lake-1", "connectionType": "DATA_LAKE"},
            {"uuid": "lake-2", "connectionType": "DATA_LAKE"},
        ]
        key = {"id": "key-id", "secret": "key-secret"}

        self._user_service.warehouses = warehouses
        self._request_wrapper.make_request_v2.return_value = MonolithResponse(
            data=Box({"key": key}), errors=None
        )

        # when
        with self.assertRaises(click.Abort):
            self._service.create(description=description, scope=scope)

        # then
        self._request_wrapper.make_request_v2.assert_not_called()
        echo.assert_called_once_with(
            "Error - Unable to resolve data lake connection: multiple lake connections found.",
            err=True,
        )

    @patch.object(click, "echo")
    def test_create_spark_key_with_no_lake(self, echo: Mock):
        # given
        description = "test key"
        scope = "spark"
        warehouses = [{"uuid": "snowflake", "connectionType": "SNOWFLAKE"}]
        key = {"id": "key-id", "secret": "key-secret"}

        self._user_service.warehouses = warehouses
        self._request_wrapper.make_request_v2.return_value = MonolithResponse(
            data=Box({"key": key}), errors=None
        )

        # when
        with self.assertRaises(click.Abort):
            self._service.create(description=description, scope=scope)

        # then
        self._request_wrapper.make_request_v2.assert_not_called()
        echo.assert_called_once_with(
            "Error - Unable to resolve data lake connection: no lake connection found.",
            err=True,
        )

    @patch.object(click, "echo")
    def test_key_deleted(self, echo: Mock):
        # given
        key_id = "key-1"

        self._request_wrapper.make_request_v2.return_value = MonolithResponse(
            data=Box({"deleted": True}), errors=None
        )

        # when
        self._service.delete(key_id)

        # then
        self._request_wrapper.make_request_v2.assert_called_once_with(
            query=Queries.delete.query,
            operation=Queries.delete.operation,
            service="integration_key_service",
            variables={"keyId": key_id},
        )

        echo.assert_called_once_with("Key has been deleted.")

    @patch.object(click, "echo")
    def test_key_not_deleted(self, echo: Mock):
        # given
        key_id = "key-1"

        self._request_wrapper.make_request_v2.return_value = MonolithResponse(
            data=Box({"deleted": False}), errors=None
        )

        # when
        self._service.delete(key_id)

        # then
        self._request_wrapper.make_request_v2.assert_called_once_with(
            query=Queries.delete.query,
            operation=Queries.delete.operation,
            service="integration_key_service",
            variables={"keyId": key_id},
        )

        echo.assert_called_once_with("Key was not deleted.")

    @patch.object(click, "echo")
    def test_get_all(self, echo: Mock):
        self._request_wrapper.make_request_v2.return_value = MonolithResponse(
            data=BoxList(
                [
                    {
                        "id": "key-id",
                        "description": "test key",
                        "scope": "spark",
                        "createdTime": "2021-01-01T00:00:00Z",
                        "createdBy": {
                            "id": "user-id",
                            "firstName": "Bilbo",
                            "lastName": "Baggins",
                            "email": "bilbo@theshire.com",
                        },
                    }
                ]
            ),
            errors=None,
        )

        # when
        self._service.get_all()

        # then
        self._request_wrapper.make_request_v2.assert_called_once_with(
            query=Queries.get_all.query,
            operation=Queries.get_all.operation,
            service="integration_key_service",
            variables={
                "scope": None,
                "resourceUuid": None,
            },
        )

        echo.assert_called_once_with(
            "╒════════╤═══════════════╤═════════╤══════════════════════╤═══════════════╕\n"
            "│ Id     │ Description   │ Scope   │ Created              │ Created By    │\n"
            "╞════════╪═══════════════╪═════════╪══════════════════════╪═══════════════╡\n"
            "│ key-id │ test key      │ spark   │ 2021-01-01T00:00:00Z │ Bilbo Baggins │\n"
            "╘════════╧═══════════════╧═════════╧══════════════════════╧═══════════════╛"
        )

        self._request_wrapper.make_request_v2.reset_mock()
        echo.reset_mock()

        # when
        self._service.get_all(scope="AirflowCallbacks")

        # then
        self._request_wrapper.make_request_v2.assert_called_once_with(
            query=Queries.get_all.query,
            operation=Queries.get_all.operation,
            service="integration_key_service",
            variables={
                "scope": "AirflowCallbacks",
                "resourceUuid": None,
            },
        )

        echo.assert_called_once_with(
            "╒════════╤═══════════════╤═════════╤══════════════════════╤═══════════════╕\n"
            "│ Id     │ Description   │ Scope   │ Created              │ Created By    │\n"
            "╞════════╪═══════════════╪═════════╪══════════════════════╪═══════════════╡\n"
            "│ key-id │ test key      │ spark   │ 2021-01-01T00:00:00Z │ Bilbo Baggins │\n"
            "╘════════╧═══════════════╧═════════╧══════════════════════╧═══════════════╛"
        )

        self._request_wrapper.make_request_v2.reset_mock()
        echo.reset_mock()

        # when
        resource_uuid = str(uuid.uuid4())
        self._service.get_all(scope="AirflowCallbacks", resource_uuid=resource_uuid)

        # then
        self._request_wrapper.make_request_v2.assert_called_once_with(
            query=Queries.get_all.query,
            operation=Queries.get_all.operation,
            service="integration_key_service",
            variables={
                "scope": "AirflowCallbacks",
                "resourceUuid": resource_uuid,
            },
        )

        echo.assert_called_once_with(
            "╒════════╤═══════════════╤═════════╤══════════════════════╤═══════════════╕\n"
            "│ Id     │ Description   │ Scope   │ Created              │ Created By    │\n"
            "╞════════╪═══════════════╪═════════╪══════════════════════╪═══════════════╡\n"
            "│ key-id │ test key      │ spark   │ 2021-01-01T00:00:00Z │ Bilbo Baggins │\n"
            "╘════════╧═══════════════╧═════════╧══════════════════════╧═══════════════╛"
        )



# Query responses
EXPECTED_GET_DATA_EXPORT_URL = "getDataExportUrl"

# Data Export Names
ALL_MONITORS_DATA_EXPORT_NAME = "ALL_MONITORS"

# Supported Schemes
FILE_SCHEME = "file"
S3_SCHEME = "s3"
SCHEME_DELIM = "://"

# Data Export headers
LIST_DATA_EXPORTS_HEADERS = ["Data Export (Name)", "Description"]



import uuid
from copy import deepcopy
from typing import Dict, Optional

import click
from box import Box
from pycarlo.core import Client

from montecarlodata.collector.validation import CollectorValidationService
from montecarlodata.config import Config
from montecarlodata.errors import echo_error, manage_errors
from montecarlodata.integrations.onboarding.base import BaseOnboardingService
from montecarlodata.integrations.onboarding.fields import (
    CONFLUENT_KAFKA_CLUSTER_TYPE,
    CONFLUENT_KAFKA_CONNECT_CLUSTER_TYPE,
    CONFLUENT_KAFKA_CONNECT_CONNECTION_TYPE,
    CONFLUENT_KAFKA_CONNECTION_TYPE,
    EXPECTED_ADD_CONFLUENT_CLUSTER_CONNECTION_RESPONSE_FIELD,
    EXPECTED_ADD_STREAMING_SYSTEM_RESPONSE_FIELD,
    EXPECTED_GET_STREAMING_SYSTEMS_RESPONSE_FIELD,
    MSK_KAFKA_CLUSTER_TYPE,
    MSK_KAFKA_CONNECT_CLUSTER_TYPE,
    MSK_KAFKA_CONNECT_CONNECTION_TYPE,
    MSK_KAFKA_CONNECTION_TYPE,
    SELF_HOSTED_KAFKA_CLUSTER_TYPE,
    SELF_HOSTED_KAFKA_CONNECT_CLUSTER_TYPE,
    SELF_HOSTED_KAFKA_CONNECT_CONNECTION_TYPE,
    SELF_HOSTED_KAFKA_CONNECTION_TYPE,
)
from montecarlodata.queries.onboarding import (
    ADD_STREAMING_CLUSTER_CONNECTION_MUTATION,
    ADD_STREAMING_SYSTEM_MUTATION,
    GET_STREAMING_SYSTEMS_QUERY,
    TEST_CONFLUENT_KAFKA_CONNECT_CRED_MUTATION,
    TEST_CONFLUENT_KAFKA_CRED_MUTATION,
    TEST_MSK_KAFKA_CONNECT_CRED_MUTATION,
    TEST_MSK_KAFKA_CRED_MUTATION,
    TEST_SELF_HOSTED_KAFKA_CONNECT_CRED_MUTATION,
    TEST_SELF_HOSTED_KAFKA_CRED_MUTATION,
)


class StreamingOnboardingService(BaseOnboardingService):
    _CONNECTION_TYPES_TO_CREDS_MUTATIONS_MAPPING = {
        CONFLUENT_KAFKA_CONNECTION_TYPE: TEST_CONFLUENT_KAFKA_CRED_MUTATION,
        CONFLUENT_KAFKA_CONNECT_CONNECTION_TYPE: TEST_CONFLUENT_KAFKA_CONNECT_CRED_MUTATION,
        MSK_KAFKA_CONNECTION_TYPE: TEST_MSK_KAFKA_CRED_MUTATION,
        MSK_KAFKA_CONNECT_CONNECTION_TYPE: TEST_MSK_KAFKA_CONNECT_CRED_MUTATION,
        SELF_HOSTED_KAFKA_CONNECTION_TYPE: TEST_SELF_HOSTED_KAFKA_CRED_MUTATION,
        SELF_HOSTED_KAFKA_CONNECT_CONNECTION_TYPE: TEST_SELF_HOSTED_KAFKA_CONNECT_CRED_MUTATION,
    }

    _CONNECTION_TYPES_TO_CLUSTER_TYPE_MAPPING = {
        CONFLUENT_KAFKA_CONNECTION_TYPE: CONFLUENT_KAFKA_CLUSTER_TYPE,
        CONFLUENT_KAFKA_CONNECT_CONNECTION_TYPE: CONFLUENT_KAFKA_CONNECT_CLUSTER_TYPE,
        MSK_KAFKA_CONNECTION_TYPE: MSK_KAFKA_CLUSTER_TYPE,
        MSK_KAFKA_CONNECT_CONNECTION_TYPE: MSK_KAFKA_CONNECT_CLUSTER_TYPE,
        SELF_HOSTED_KAFKA_CONNECTION_TYPE: SELF_HOSTED_KAFKA_CLUSTER_TYPE,
        SELF_HOSTED_KAFKA_CONNECT_CONNECTION_TYPE: SELF_HOSTED_KAFKA_CONNECT_CLUSTER_TYPE,
    }

    def __init__(self, config: Config, mc_client: Client, command_name: str, **kwargs):
        super().__init__(config, command_name=command_name, **kwargs)
        self._validation_service = CollectorValidationService(
            config=config,
            mc_client=mc_client,
            user_service=self._user_service,
            request_wrapper=self._request_wrapper,
            command_name=self._command_name,
        )

    @manage_errors
    def create_streaming_system(
        self,
        streaming_system_type: str,
        streaming_system_name: str,
        dc_id: Optional[str] = None,
    ) -> None:
        """
        Creates a streaming system.
        """
        response = self._request_wrapper.make_request_v2(
            query=ADD_STREAMING_SYSTEM_MUTATION,
            operation=EXPECTED_ADD_STREAMING_SYSTEM_RESPONSE_FIELD,
            service="streaming_onboarding_service",
            variables=dict(
                streaming_system_type=streaming_system_type,
                streaming_system_name=streaming_system_name,
                dc_id=dc_id,
            ),
        )
        streaming_system = response.data.streamingSystem  # type: ignore
        click.echo(
            f"Successfully created the {streaming_system.type} streaming system "
            f"{streaming_system.name} with uuid: {streaming_system.uuid}."
        )

    def get_streaming_system(self, streaming_system_uuid: str) -> Optional[Box]:
        """
        Gets a streaming system.
        """
        response = self._request_wrapper.make_request_v2(
            query=GET_STREAMING_SYSTEMS_QUERY,
            operation=EXPECTED_GET_STREAMING_SYSTEMS_RESPONSE_FIELD,
            service="streaming_onboarding_service",
            variables=dict(include_clusters=False),
        )
        for entry in response.data:  # type: ignore
            system = entry.system
            if system.uuid == streaming_system_uuid:
                return system

        echo_error(f"cannot find the stream system with uuid {streaming_system_uuid}")

    @manage_errors
    def test_new_confluent_kafka_credentials(
        self,
        cluster: str,
        api_key: str,
        secret: str,
        url: str,
        dc_id: Optional[uuid.UUID] = None,
    ) -> Optional[str]:
        return self._validation_service.test_new_credentials(
            connection_type=CONFLUENT_KAFKA_CONNECTION_TYPE,
            dc_id=dc_id,
            cluster=cluster,
            api_key=api_key,
            secret=secret,
            url=url,
        )

    @manage_errors
    def test_new_confluent_kafka_connect_credentials(
        self,
        confluent_env: str,
        cluster: str,
        api_key: str,
        secret: str,
        url: Optional[str] = None,
        dc_id: Optional[uuid.UUID] = None,
    ) -> Optional[str]:
        return self._validation_service.test_new_credentials(
            dc_id=dc_id,
            connection_type=CONFLUENT_KAFKA_CONNECT_CONNECTION_TYPE,
            confluent_env=confluent_env,
            cluster=cluster,
            api_key=api_key,
            secret=secret,
            url=url,
        )

    @manage_errors
    def test_new_msk_kafka_credentials(
        self,
        cluster: str,
        url: str,
        auth_type: str,
        auth_token: Optional[str] = None,
        dc_id: Optional[uuid.UUID] = None,
    ) -> Optional[str]:
        return self._validation_service.test_new_credentials(
            dc_id=dc_id,
            connection_type=MSK_KAFKA_CONNECTION_TYPE,
            cluster=cluster,
            auth_type=auth_type,
            auth_token=auth_token,
            url=url,
        )

    @manage_errors
    def test_new_msk_kafka_connect_credentials(
        self,
        cluster_arn: str,
        iam_role_arn: str,
        external_id: Optional[str] = None,
        dc_id: Optional[uuid.UUID] = None,
    ) -> Optional[str]:
        return self._validation_service.test_new_credentials(
            dc_id=dc_id,
            connection_type=MSK_KAFKA_CONNECT_CONNECTION_TYPE,
            cluster_arn=cluster_arn,
            iam_role_arn=iam_role_arn,
            external_id=external_id,
        )

    @manage_errors
    def test_new_self_hosted_kafka_credentials(
        self,
        cluster: str,
        url: str,
        auth_type: str,
        auth_token: Optional[str] = None,
        dc_id: Optional[uuid.UUID] = None,
    ) -> Optional[str]:
        return self._validation_service.test_new_credentials(
            dc_id=dc_id,
            connection_type=SELF_HOSTED_KAFKA_CONNECTION_TYPE,
            cluster=cluster,
            auth_type=auth_type,
            auth_token=auth_token,
            url=url,
        )

    @manage_errors
    def test_new_self_hosted_kafka_connect_credentials(
        self,
        cluster: str,
        url: str,
        auth_type: str,
        auth_token: Optional[str] = None,
        dc_id: Optional[uuid.UUID] = None,
    ) -> Optional[str]:
        return self._validation_service.test_new_credentials(
            dc_id=dc_id,
            connection_type=SELF_HOSTED_KAFKA_CONNECT_CONNECTION_TYPE,
            cluster=cluster,
            auth_type=auth_type,
            auth_token=auth_token,
            url=url,
        )

    @manage_errors
    def onboard_streaming_cluster_connection(self, **kwargs) -> None:
        connection_type = kwargs.get("connection_type")
        if connection_type not in self._CONNECTION_TYPES_TO_CREDS_MUTATIONS_MAPPING.keys():
            echo_error("Not supported streaming connection type")
            click.Abort()

        streaming_system_id = kwargs.get("streaming_system_id")
        streaming_system = None
        if streaming_system_id:
            # if the streaming system exist, then we'll proceed, otherwise, we'd stop.
            # that's because either the streaming system doesn't exist or it doesn't
            # belong to the user's account.
            streaming_system = self.get_streaming_system(streaming_system_uuid=streaming_system_id)

        credential_key = kwargs.get("key")
        if not credential_key:
            # copy params for creating credential key
            params_copy = deepcopy(kwargs)
            if not params_copy.get("dc_id") and params_copy.get("streaming_system_id"):
                params_copy["dc_id"] = streaming_system.dcId  # type: ignore
            credential_key = self._test_and_generate_credential_key(
                connection_type=connection_type,  # type: ignore
                params=params_copy,
            )
            if credential_key:
                kwargs["key"] = credential_key
            else:
                # credentials testing fail or user doesn't want to continue
                click.Abort()

        if not kwargs.get("mc_cluster_id"):
            # for new cluster, we'll add the cluster type here.
            kwargs["new_cluster_type"] = self._CONNECTION_TYPES_TO_CLUSTER_TYPE_MAPPING[
                connection_type
            ]  # type: ignore

        response = self._request_wrapper.make_request_v2(
            query=ADD_STREAMING_CLUSTER_CONNECTION_MUTATION,
            operation=EXPECTED_ADD_CONFLUENT_CLUSTER_CONNECTION_RESPONSE_FIELD,
            service="streaming_onboarding_service",
            variables=kwargs,
        )

        connection = response.data.connection  # type: ignore
        click.echo(
            f"Successfully created the {connection.type} connection {connection.uuid} for cluster "
            f"{connection.streamingCluster.uuid}, in streaming system "
            f"{connection.streamingCluster.streamingSystemUuid}."
        )

    def _test_and_generate_credential_key(
        self,
        connection_type: str,
        params: Dict,
    ) -> Optional[str]:
        parameters: Dict = {
            "cluster": params.get("new_cluster_id"),
        }
        if connection_type == CONFLUENT_KAFKA_CONNECTION_TYPE:
            parameters["url"] = params.get("url")
            parameters["api_key"] = params.get("api_key")
            parameters["secret"] = params.get("secret")
        elif connection_type == CONFLUENT_KAFKA_CONNECT_CONNECTION_TYPE:
            parameters["confluent_env"] = params.get("confluent_env")
            parameters["api_key"] = params.get("api_key")
            parameters["secret"] = params.get("secret")
            if "url" in params:
                parameters["url"] = params.get("url")
        elif connection_type == MSK_KAFKA_CONNECT_CONNECTION_TYPE:
            parameters["cluster_arn"] = params.get("cluster_arn")
            parameters["iam_role_arn"] = params.get("iam_role_arn")
            if "external_id" in params:
                parameters["external_id"] = params.get("external_id")
        elif connection_type in [
            MSK_KAFKA_CONNECTION_TYPE,
            SELF_HOSTED_KAFKA_CONNECTION_TYPE,
            SELF_HOSTED_KAFKA_CONNECT_CONNECTION_TYPE,
        ]:
            parameters["url"] = params.get("url")
            parameters["auth_type"] = params.get("auth_type")
            parameters["auth_token"] = params.get("auth_token")
        else:
            click.echo(f"Not supported connection type {connection_type}")

        return self._validation_service.test_new_credentials(
            connection_type=connection_type,
            dc_id=params.get("dc_id"),
            **parameters,
        )



from typing import Dict, List, Optional

import click
import click_config_file

from montecarlodata import settings
from montecarlodata.common.commands import DC_RESOURCE_OPTIONS, DISAMBIGUATE_DC_OPTIONS
from montecarlodata.common.resources import CloudResourceService
from montecarlodata.discovery.networking import NetworkDiscoveryService
from montecarlodata.discovery.policy_gen import PolicyDiscoveryService
from montecarlodata.tools import add_common_options

# Shared command verbiage
DISCOVERY_REVIEW_VERBIAGE = (
    "After review, output of this command can be redirected into "
    "`montecarlo integrations create-role` or `montecarlo discovery cf-role-gen` "
    "if you prefer IaC"
)

GLUE_COMMON_OPTIONS = [
    click.option(
        "--database-name",
        "database_names",
        required=True,
        multiple=True,
        help=(
            "Glue/Athena database name to generate a policy from. Enter '\\*' to give "
            "Monte Carlo access to all databases. This option can be passed multiple "
            "times for more than one database."
        ),
    ),
    click.option(
        "--data-bucket-name",
        "bucket_names",
        required=False,
        multiple=True,
        help=(
            "Name of a S3 bucket storing the data for your Glue/Athena tables. If this option is "
            "not specified the bucket names are derived (looked up) from the tables in your "
            "databases. This option can be passed multiple times for more than one bucket. "
            "Enter '\\*' to give Monte Carlo access to all buckets."
        ),
    ),
]


@click.group(help="Display information about resources.")
def discovery():
    """
    Group for any discovery related subcommands
    """
    pass


@discovery.command(help="List details about EMR clusters in a region.")
@click.option("--aws-profile", help="AWS profile.", required=True)
@click.option("--aws-region", help="AWS region.", required=True)
@click.option(
    "--only-log-locations",
    help="Display only unique log locations",
    is_flag=True,
    default=False,
)
@click.option(
    "--created-after",
    help="Display clusters created after date (e.g. 2017-07-04T00:01:30)",
    required=False,
)
@click.option(
    "--state",
    help="Cluster states",
    required=False,
    type=click.Choice(["active", "terminated", "failed"]),
    multiple=True,
)
@click.option(
    "--no-grid",
    help="Do not display as grid and print as results are available, "
    "useful when the cluster list is large",
    is_flag=True,
    default=False,
)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
@click.pass_obj
def list_emr_clusters(
    ctx: Dict,
    aws_profile: Optional[str] = None,
    aws_region: Optional[str] = None,
    only_log_locations: bool = False,
    created_after: Optional[str] = None,
    state: Optional[List] = None,
    no_grid: bool = False,
) -> None:
    CloudResourceService(
        config=ctx["config"],
        command_name="discovery list_emr_clusters",
        aws_profile_override=aws_profile,
        aws_region_override=aws_region,
    ).list_emr_clusters(
        only_log_locations=only_log_locations,
        created_after=created_after,
        states=state,
        no_grid=no_grid,
    )


@discovery.command(
    help="Alpha network recommender. Attempts to analyze and makes recommendations "
    "on how to connect a resource with the Data Collector."
)
@click.pass_obj
@click.option(
    "--resource-identifier",
    required=True,
    help="Identifier for the AWS resource you want to connect the Collector with "
    "(e.g. Redshift cluster ID).",
)
@click.option(
    "--resource-type",
    required=True,
    help="Type of AWS resource.",
    type=click.Choice(list(NetworkDiscoveryService.MCD_NETWORK_REC_RESOURCE_TYPE_MAP.keys())),
)
@click.option(
    "--collector-aws-profile",
    required=True,
    help="AWS profile for the Collector.",
)
@add_common_options(DC_RESOURCE_OPTIONS)
@add_common_options(DISAMBIGUATE_DC_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def network_recommender(ctx, **kwargs):
    NetworkDiscoveryService(
        config=ctx["config"],
        command_name="discovery network_recommender",
        aws_wrapper=None,
    ).recommend_network_dispatcher(**kwargs)


@discovery.command(
    help="Generate a CloudFormation template to create a resource access IAM role. "
    "After review, this template can be deployed using CloudFormation. "
    "The Role ARN and External ID for onboarding can be found in the stack outputs."
)
@click.pass_obj
@click.option(
    "--policy-file",
    "policy_files",
    required=True,
    multiple=True,
    type=click.File(),
    help="File containing an IAM policy to generate an IAM role from. This option can be passed "
    "multiple times for more than one policy.",
)
@add_common_options(DISAMBIGUATE_DC_OPTIONS)
def cf_role_gen(ctx, **kwargs):
    PolicyDiscoveryService(
        config=ctx["config"],
        command_name="discovery cf_role_gen",
        aws_wrapper=None,
    ).generate_cf_role(**kwargs)


@discovery.command(help=f"Generate an IAM policy for Glue. {DISCOVERY_REVIEW_VERBIAGE}.")
@click.pass_obj
@add_common_options(GLUE_COMMON_OPTIONS)
@add_common_options(DC_RESOURCE_OPTIONS)
@add_common_options(DISAMBIGUATE_DC_OPTIONS)
def glue_policy_gen(ctx, **kwargs):
    PolicyDiscoveryService(
        config=ctx["config"],
        command_name="discovery glue_policy_gen",
        aws_wrapper=None,
    ).generate_glue_policy(**kwargs)


@discovery.command(help=f"Generate an IAM policy for MSK. {DISCOVERY_REVIEW_VERBIAGE}.")
@click.pass_obj
@add_common_options(DC_RESOURCE_OPTIONS)
@add_common_options(DISAMBIGUATE_DC_OPTIONS)
def msk_policy_gen(ctx, **kwargs):
    PolicyDiscoveryService(
        config=ctx["config"],
        command_name="discovery msk_policy_gen",
        aws_wrapper=None,
    ).generate_msk_policy(**kwargs)


@discovery.command(help=f"Generate an IAM policy for Athena. {DISCOVERY_REVIEW_VERBIAGE}.")
@click.pass_obj
@add_common_options(GLUE_COMMON_OPTIONS)
@click.option(
    "--workgroup-name",
    required=True,
    default="primary",
    show_default=True,
    help="Athena workgroup for Monte Carlo to use when performing queries. "
    'The "primary" workgroup for the region is used if one is not specified.',
)
@add_common_options(DC_RESOURCE_OPTIONS)
@add_common_options(DISAMBIGUATE_DC_OPTIONS)
def athena_policy_gen(ctx, **kwargs):
    PolicyDiscoveryService(
        config=ctx["config"],
        command_name="discovery athena_policy_gen",
        aws_wrapper=None,
    ).generate_athena_policy(**kwargs)



import uuid
from unittest import TestCase
from unittest.mock import ANY, Mock, call, patch

import click
from box import Box
from pycarlo.core import Client

from montecarlodata.platform.service import PlatformService
from montecarlodata.queries.platform import (
    MUTATION_TRIGGER_CAAS_MIGRATION_TEST,
    QUERY_CAAS_MIGRATION_TEST_STATUS,
    QUERY_GET_SERVICES,
)

_DC_CLOUD_V2 = Box(uuid=str(uuid.uuid4()), deployment_type="CLOUD_V2", active=True, agents=[])
_DC_WITH_AGENT = Box(
    uuid=str(uuid.uuid4()),
    deployment_type="REMOTE_V1_5",
    active=True,
    agents=[
        Box(
            is_deleted=True,
            endpoint="deleted_agent_endpoint",
        ),
        Box(
            is_deleted=False,
            endpoint="agent_endpoint",
        ),
    ],
)


class PlatformServiceTest(TestCase):
    def setUp(self) -> None:
        self._mc_client_mock = Mock(autospec=Client)
        self._service = PlatformService(mc_client=self._mc_client_mock, command_name="test")

    @patch("montecarlodata.platform.service.tabulate")
    def test_platform_list(self, tabulate_mock):
        self._mc_client_mock.return_value = Box(
            get_user=Box(account=Box(data_collectors=[_DC_CLOUD_V2, _DC_WITH_AGENT]))
        )
        self._service.list_services()
        self._mc_client_mock.assert_called_once()
        tabulate_mock.assert_called_once_with(
            [
                [str(_DC_CLOUD_V2.uuid), "CLOUD_V2", "N/A"],
                [str(_DC_WITH_AGENT.uuid), "REMOTE_V1_5", "agent_endpoint"],
            ],
            headers=["Service ID", "Deployment Type", "Endpoint"],
            tablefmt="fancy_grid",
        )

    @patch("montecarlodata.platform.service.click.echo")
    def test_trigger_migration_test_not_needed(self, mock_click_echo):
        self._mc_client_mock.return_value = Box(
            get_user=Box(
                account=Box(
                    data_collectors=[_DC_CLOUD_V2],
                ),
            ),
        )
        with self.assertRaises(click.Abort):
            self._service.caas_migration_test(None)
        mock_click_echo.assert_called_once_with("No need to migrate CLOUD_V2 services.")

    @patch("montecarlodata.platform.service.click.echo")
    def test_trigger_migration_test_dc_id_required(self, mock_click_echo):
        self._mc_client_mock.return_value = Box(
            get_user=Box(
                account=Box(
                    data_collectors=[_DC_CLOUD_V2, _DC_WITH_AGENT],
                ),
            ),
        )
        with self.assertRaises(click.Abort):
            self._service.caas_migration_test(None)
        mock_click_echo.assert_called_once_with(
            "There are multiple active services, please specify one using --service-id."
        )

    @patch("montecarlodata.platform.service.click.echo")
    def test_trigger_migration_test_no_dcs(self, mock_click_echo):
        self._mc_client_mock.return_value = Box(
            get_user=Box(
                account=Box(
                    data_collectors=[],
                ),
            ),
        )
        with self.assertRaises(click.Abort):
            self._service.caas_migration_test(None)
        mock_click_echo.assert_called_once_with("There are no active services in this account.")

    @patch("montecarlodata.platform.service.click.echo")
    def test_trigger_migration_test_wrong_dc_ids(self, mock_click_echo):
        self._mc_client_mock.return_value = Box(
            get_user=Box(
                account=Box(
                    data_collectors=[_DC_CLOUD_V2],
                ),
            ),
        )
        with self.assertRaises(click.Abort):
            self._service.caas_migration_test(uuid.uuid4())
        mock_click_echo.assert_called_once_with(
            "No active service found with the specified ID, use "
            "`montecarlo platform list` to list the active services."
        )

    @patch("montecarlodata.platform.service.time.sleep")
    @patch("montecarlodata.platform.service.tabulate")
    def test_trigger_migration_multiple_dcs(self, mock_tabulate, mock_sleep):
        migration_uuid = str(uuid.uuid4())
        self._mc_client_mock.side_effect = [
            Box(
                get_user=Box(
                    account=Box(
                        data_collectors=[_DC_CLOUD_V2, _DC_WITH_AGENT],
                    ),
                ),
            ),
            Box(
                trigger_platform_migration_test=Box(
                    migration_uuid=migration_uuid,
                )
            ),
            Box(
                get_platform_migration_status=Box(
                    output=Box(
                        status="dry_run_completed",
                        success=True,
                        error_message=None,
                        tested_connections=[
                            Box(
                                name="connection_name",
                                uuid="connection_uuid",
                                connection_type="snowflake",
                                success=True,
                                error_message=None,
                            )
                        ],
                        tested_agent=None,
                    ),
                ),
            ),
        ]
        self._service.caas_migration_test(_DC_WITH_AGENT.uuid)
        mock_tabulate.assert_called_once_with(
            [
                [
                    "connection_name",
                    "connection_uuid",
                    "snowflake",
                    "Success",
                    "None",
                    "",
                ]
            ],
            headers=[
                "Name",
                "Connection UUID",
                "Connection Type",
                "Result",
                "Required Action",
                "Error Message",
            ],
            tablefmt="fancy_grid",
            maxcolwidths=ANY,
        )

    @patch("montecarlodata.platform.service.time.sleep")
    @patch("montecarlodata.platform.service.tabulate")
    def test_trigger_migration_single_dc(self, mock_tabulate, mock_sleep):
        migration_uuid = str(uuid.uuid4())
        self._mc_client_mock.side_effect = [
            Box(
                get_user=Box(
                    account=Box(
                        data_collectors=[_DC_WITH_AGENT],
                    ),
                ),
            ),
            Box(
                trigger_platform_migration_test=Box(
                    migration_uuid=migration_uuid,
                )
            ),
            Box(
                get_platform_migration_status=Box(
                    output=Box(
                        status="dry_run_completed",
                        success=True,
                        error_message=None,
                        tested_connections=[
                            Box(
                                name="connection_name",
                                uuid="connection_uuid",
                                connection_type="snowflake",
                                success=True,
                                error_message=None,
                            )
                        ],
                        tested_agent=None,
                    ),
                ),
            ),
        ]
        self._service.caas_migration_test(None)
        self._mc_client_mock.assert_has_calls(
            [
                call(
                    QUERY_GET_SERVICES,
                    additional_headers={
                        "x-mcd-telemetry-reason": "cli",
                        "x-mcd-telemetry-service": "platform_service",
                        "x-mcd-telemetry-command": "test",
                    },
                ),
                call(
                    MUTATION_TRIGGER_CAAS_MIGRATION_TEST,
                    variables={
                        "dcUuid": str(_DC_WITH_AGENT.uuid),
                    },
                    additional_headers={
                        "x-mcd-telemetry-reason": "cli",
                        "x-mcd-telemetry-service": "platform_service",
                        "x-mcd-telemetry-command": "test",
                    },
                ),
                call(
                    QUERY_CAAS_MIGRATION_TEST_STATUS,
                    variables={
                        "dcUuid": str(_DC_WITH_AGENT.uuid),
                        "migrationUuid": migration_uuid,
                    },
                    additional_headers={
                        "x-mcd-telemetry-reason": "cli",
                        "x-mcd-telemetry-service": "platform_service",
                        "x-mcd-telemetry-command": "test",
                    },
                ),
            ]
        )
        mock_tabulate.assert_called_once_with(
            [
                [
                    "connection_name",
                    "connection_uuid",
                    "snowflake",
                    "Success",
                    "None",
                    "",
                ]
            ],
            headers=[
                "Name",
                "Connection UUID",
                "Connection Type",
                "Result",
                "Required Action",
                "Error Message",
            ],
            tablefmt="fancy_grid",
            maxcolwidths=ANY,
        )



from dataclasses import dataclass


@dataclass
class DataExport:
    name: str
    title: str
    description: str


DATA_EXPORT_CATALOG = [
    DataExport(
        name="ALL_MONITORS",
        title="All monitors",
        description="All monitors with aggregated properties, excluding deleted monitors.",
    ),
]



from unittest import TestCase
from unittest.mock import Mock, patch

import click.exceptions

from montecarlodata.common.user import UserService
from montecarlodata.config import Config
from montecarlodata.keys.airflow import AirflowService


class AirflowKeysTests(TestCase):
    config = Config(
        mcd_id="12345",
        mcd_token="67890",
        mcd_api_endpoint="https://api.getmontecarlo.com/graphql",
    )

    def setUp(self) -> None:
        self._user_service = Mock(autospec=UserService)
        self._service = AirflowService(
            config=self.config,
            command_name="test",
            user_service=self._user_service,
        )

    def test_get_resource_id(self):
        self._user_service.etl_containers = [
            {
                "name": "airflow_1",
                "uuid": "1234",
                "connections": [
                    {"type": "AIRFLOW"},
                ],
            },
        ]
        rid = self._service.get_airflow_resource_uuid()
        self.assertEqual("1234", rid)

    @patch("click.echo")
    def test_get_resource_id_no_etl_containers(self, mock_echo):
        self._user_service.etl_containers = []
        with self.assertRaises(click.exceptions.Abort):
            self._service.get_airflow_resource_uuid()
        mock_echo.assert_called_with("Error - No Airflow connection found", err=True)

    @patch("click.echo")
    def test_get_resource_id_no_airflow_containers(self, mock_echo):
        self._user_service.etl_containers = [
            {
                "name": "power_bi",
                "uuid": "1234",
                "connections": [
                    {"type": "POWER_BI"},
                ],
            },
        ]
        with self.assertRaises(click.exceptions.Abort):
            self._service.get_airflow_resource_uuid()
        mock_echo.assert_called_with("Error - No Airflow connection found", err=True)

    @patch("click.echo")
    def test_get_resource_multiple_etl_containers(self, mock_echo):
        self._user_service.etl_containers = [
            {
                "name": "airflow_1",
                "uuid": "1234",
                "connections": [
                    {"type": "AIRFLOW"},
                ],
            },
            {
                "name": "airflow_2",
                "uuid": "1235",
                "connections": [
                    {"type": "AIRFLOW"},
                ],
            },
        ]
        with self.assertRaises(click.exceptions.Abort):
            self._service.get_airflow_resource_uuid()
        mock_echo.assert_called_with(
            "Error - Multiple Airflow connections found, use --name to disambiguate",
            err=True,
        )

        rid = self._service.get_airflow_resource_uuid(name="airflow_1")
        self.assertEqual("1234", rid)

    @patch("click.echo")
    def test_get_resource_id_name_not_found(self, mock_echo):
        self._user_service.etl_containers = [
            {
                "name": "airflow_1",
                "uuid": "1234",
                "connections": [
                    {"type": "AIRFLOW"},
                ],
            },
        ]
        with self.assertRaises(click.exceptions.Abort):
            self._service.get_airflow_resource_uuid(name="invalid")
        mock_echo.assert_called_with(
            "Error - No Airflow connection found with name invalid", err=True
        )

    @patch("click.echo")
    def test_get_resource_id_multiple_same_name(self, mock_echo):
        self._user_service.etl_containers = [
            {
                "name": "airflow_1",
                "uuid": "1234",
                "connections": [
                    {"type": "AIRFLOW"},
                ],
            },
            {
                "name": "airflow_1",
                "uuid": "1234",
                "connections": [
                    {"type": "AIRFLOW"},
                ],
            },
        ]
        with self.assertRaises(click.exceptions.Abort):
            self._service.get_airflow_resource_uuid(name="airflow_1")
        mock_echo.assert_called_with(
            "Error - Multiple Airflow connections found with name airflow_1", err=True
        )



import copy
import json
from io import StringIO
from unittest import TestCase
from unittest.mock import ANY, MagicMock, Mock, call, patch

import click
from box import Box, BoxList

from montecarlodata.common.data import (
    AWSArn,
    BucketRow,
    DcResourceProperties,
    EventProperties,
    ResourceProperties,
)
from montecarlodata.common.resources import CloudResourceService
from montecarlodata.common.user import UserService
from montecarlodata.utils import AwsClientWrapper
from tests.helpers import capture_function
from tests.test_common_user import _SAMPLE_CONFIG

_SAMPLE_REGION = "us-east-1"
_SAMPLE_BUCKET_NAME = "bucket"
_SAMPLE_BASE_OPTIONS = {"foo": "bar"}
_SAMPLE_DC_OUTPUT = [
    {
        "OutputKey": "PrivateS3BucketArn",
        "OutputValue": f"arn:aws:s3:::{_SAMPLE_BUCKET_NAME}",
    }
]
_SAMPLE_CONNECTION_PATH = "test"
_SAMPLE_ACCOUNT_ID = "1234"
_SAMPLE_EXTERNAL_ID = "5678"
_SAMPLE_TIME = "ts"
_SAMPLE_POLICY_FILE = {"hello": "world"}
_SAMPLE_TRUST_POLICY = {
    "Version": "2012-10-17",
    "Statement": [
        {
            "Effect": "Allow",
            "Principal": {"AWS": f"arn:aws:iam::{_SAMPLE_ACCOUNT_ID}:root"},
            "Action": "sts:AssumeRole",
            "Condition": {"StringEquals": {"sts:ExternalId": _SAMPLE_EXTERNAL_ID}},
        }
    ],
}
_SAMPLE_EMR_NAME = "name"
_SAMPLE_EMR_ID = "cluster_id"
_SAMPLE_EMR_ID_2 = "cluster_id_2"
_SAMPLE_EMR_STATE = "WAITING"
_SAMPLE_EMR_STATUS = {"State": _SAMPLE_EMR_STATE}
_SAMPLE_EMR_LOG_URI = "s3://bucket/path"
_SAMPLE_EMR_LOG_URI_2 = "s3://bucket_2/path"

_SAMPLE_BUCKETS_FILE = """bucket_name,prefix,suffix
foo,,
bar,prefix1,
baz,,suffix1
qux,prefix2,suffix2
"""

_SAMPLE_EVENT_PAYLOAD = {
    "bucket_name": "mcd-test-events",
    "collector_aws_profile": "dev",
    "prefix": None,
    "suffix": None,
    "topic_arn": None,
    "bucket_aws_profile": None,
    "event_type": "metadata",
}
_SAMPLE_BUCKETS_FILE_EVENT_PAYLOAD = {
    "buckets_filename": "buckets.csv",
    "collector_aws_profile": "dev",
    "prefix": None,
    "suffix": None,
    "topic_arn": None,
    "bucket_aws_profile": None,
    "event_type": "query-logs",
}
_SAMPLE_COLLECTOR = {
    "uuid": "e34eaf86-092d-47ac-97c3-cfa8b6a6e7fe",
    "stackArn": "arn:aws:cloudformation:us-east-1:123456789:stack/loki/99",
    "active": True,
    "customerAwsAccountId": "123456789",
    "templateVersion": "42",
    "codeVersion": "2042",
    "lastUpdated": None,
}

_SAMPLE_DC_CLIENT = Mock()
_SAMPLE_CUST_RESOURCES_CLIENT = Mock()

_SAMPLE_RESOURCE_PROPERTY_1 = ResourceProperties(
    bucket_name="mcd-test-events",
    client=copy.deepcopy(_SAMPLE_CUST_RESOURCES_CLIENT),
    account_id="123456789",
    region="us-east-1",
)
_SAMPLE_RESOURCE_PROPERTY_2 = ResourceProperties(
    bucket_name="mcd-test-events-us-east-2",
    client=copy.deepcopy(_SAMPLE_CUST_RESOURCES_CLIENT),
    account_id="123456789",
    region="us-east-2",
)

_SAMPLE_RESOURCE_PROPERTIES = [_SAMPLE_RESOURCE_PROPERTY_1, _SAMPLE_RESOURCE_PROPERTY_2]
_SAMPLE_EVENT_DETAILS = EventProperties(
    event_type="metadata",
    event_queue_arn="arn:aws:sqs:us-east-1:123456789:loki-MetadataEventQueue-42",
    collection_region="us-east-1",
    collection_account_id="123456789",
    collection_client=_SAMPLE_DC_CLIENT,
    resource_properties=_SAMPLE_RESOURCE_PROPERTIES,
)


class CloudResourcesTest(TestCase):
    def setUp(self) -> None:
        self._user_service_mock = Mock(autospec=UserService)
        self._aws_wrapper_mock = Mock(autospec=AwsClientWrapper)

        self._service = CloudResourceService(
            _SAMPLE_CONFIG,
            command_name="test",
            aws_wrapper=self._aws_wrapper_mock,
            user_service=self._user_service_mock,
        )

    def tearDown(self) -> None:
        _SAMPLE_DC_CLIENT.reset_mock()
        _SAMPLE_CUST_RESOURCES_CLIENT.reset_mock()

    @patch.object(CloudResourceService, "_read_json")
    def test_create_role_with_invalid_policy(self, mock_read):
        mock_read.side_effect = json.decoder.JSONDecodeError
        self._user_service_mock.active_collector = {"customerAwsAccountId": _SAMPLE_ACCOUNT_ID}

        with self.assertRaises(click.exceptions.Abort):
            self._service.create_role(path_to_policy_doc="invalid")

    @patch.object(CloudResourceService, "_read_json")
    def test_create_role_with_missing_values(self, mock_read):
        mock_read.return_value = _SAMPLE_POLICY_FILE
        self._user_service_mock.active_collector = {}
        with self.assertRaises(click.exceptions.Abort):
            self._service.create_role(path_to_policy_doc="foo")

    @patch.object(CloudResourceService, "_read_json")
    @patch.object(CloudResourceService, "_generate_random_token")
    @patch("time.time", MagicMock(return_value=_SAMPLE_TIME))
    def test_create_role(self, mock_token, mock_read):
        path_to_policy_doc = "foo"
        mock_read.return_value = _SAMPLE_POLICY_FILE
        mock_token.return_value = _SAMPLE_EXTERNAL_ID
        sample_collector = {
            **_SAMPLE_COLLECTOR,
            "customerAwsAccountId": _SAMPLE_ACCOUNT_ID,
            "agents": [],
        }
        self._user_service_mock.active_collector = sample_collector
        self._user_service_mock.get_collector.return_value = Box(sample_collector)
        self._user_service_mock.get_collector_agent.return_value = None

        self._service.create_role(path_to_policy_doc=path_to_policy_doc)

        mock_read.assert_called_once_with(path_to_policy_doc)
        mock_token.assert_called_once_with()

        self._aws_wrapper_mock.create_role.assert_called_once_with(
            role_name=f"monte-carlo-integration-role-{_SAMPLE_TIME}",
            trust_policy=json.dumps(_SAMPLE_TRUST_POLICY),
            tags=self._service._MCD_TAGS,
        )

        self._aws_wrapper_mock.attach_inline_policy.assert_called_once_with(
            role_name=f"monte-carlo-integration-role-{_SAMPLE_TIME}",
            policy_name=f"monte-carlo-integration-cli-policy-{_SAMPLE_TIME}",
            policy_doc=json.dumps(_SAMPLE_POLICY_FILE),
        )

    def test_generate_policy(self):
        policy = self._service._generate_trust_policy(
            account_id=_SAMPLE_ACCOUNT_ID, external_id=_SAMPLE_EXTERNAL_ID
        )

        self.assertEqual(policy, json.dumps(_SAMPLE_TRUST_POLICY))

    @patch.object(click, "echo")
    def test_get_aws_account_id(self, mock_echo):
        aws_remote_agent_data = {
            "uuid": "remote-aws-agent",
            "agentType": "REMOTE_AGENT",
            "platform": "AWS",
            "storageType": "S3",
            "endpoint": "arn:aws:lambda:us-east-1:210987654321:function:mcd-agent-Lambda",
            "isDeleted": False,
            "imageVersion": "800",
        }
        self._user_service_mock.get_collector.return_value = Box(
            {**_SAMPLE_COLLECTOR, "agents": [aws_remote_agent_data]}
        )
        self._user_service_mock.get_collector_agent.return_value = Box(aws_remote_agent_data)
        self.assertEqual("210987654321", self._service._get_aws_account_id())
        mock_echo.assert_not_called()

        self._user_service_mock.get_collector.reset_mock()
        self._user_service_mock.get_collector_agent.reset_mock()
        mock_echo.reset_mock()

        gcp_remote_agent_data = {
            "uuid": "remote-gcp-agent",
            "agentType": "REMOTE_AGENT",
            "platform": "GCP",
            "storageType": "GCS",
            "endpoint": "https://example.com",
            "isDeleted": False,
            "imageVersion": "800",
        }
        self._user_service_mock.get_collector.return_value = Box(
            {**_SAMPLE_COLLECTOR, "agents": [gcp_remote_agent_data]}
        )
        self._user_service_mock.get_collector_agent.return_value = Box(gcp_remote_agent_data)
        with self.assertRaises(click.exceptions.Abort):
            self._service._get_aws_account_id()
        mock_echo.assert_called_once_with(
            "Error - The collector is configured with a non-AWS agent", err=True
        )

        self._user_service_mock.get_collector.reset_mock()
        self._user_service_mock.get_collector_agent.reset_mock()
        mock_echo.reset_mock()

        s3_data_store_agent_data = {
            "uuid": "s3-data-store-agent",
            "agentType": "DATA_STORE_AGENT",
            "platform": "AWS",
            "storageType": "S3",
            "endpoint": "s3-bucket",
            "isDeleted": False,
            "imageVersion": "800",
        }
        self._user_service_mock.get_collector.return_value = Box(
            {**_SAMPLE_COLLECTOR, "agents": [s3_data_store_agent_data]}
        )
        self._user_service_mock.get_collector_agent.return_value = Box(s3_data_store_agent_data)
        self.assertEqual("123456789", self._service._get_aws_account_id())
        mock_echo.assert_not_called()

        self._user_service_mock.get_collector.reset_mock()
        self._user_service_mock.get_collector_agent.reset_mock()
        mock_echo.reset_mock()

        self._user_service_mock.get_collector.return_value = Box(
            {**_SAMPLE_COLLECTOR, "agents": []}
        )
        self._user_service_mock.get_collector_agent.return_value = None
        self.assertEqual("123456789", self._service._get_aws_account_id())
        mock_echo.assert_not_called()

    def test_list_emr_clusters(self):
        self._aws_wrapper_mock.get_emr_cluster_details.return_value = [
            {
                "Name": _SAMPLE_EMR_NAME,
                "Id": _SAMPLE_EMR_ID,
                "Status": _SAMPLE_EMR_STATUS,
                "LogUri": _SAMPLE_EMR_LOG_URI,
            }
        ]

        std_out = capture_function(
            self._service.list_emr_clusters, {"table_format": "plain"}
        ).std_out

        expected_output = (
            "Name    Id          State    LogUri\n"
            f"{_SAMPLE_EMR_NAME}    {_SAMPLE_EMR_ID}  {_SAMPLE_EMR_STATE}  {_SAMPLE_EMR_LOG_URI}"
        )
        self.assertEqual(std_out.getvalue().strip(), expected_output)

    def test_list_emr_clusters_no_wait(self):
        self._aws_wrapper_mock.get_emr_cluster_details.return_value = [
            {
                "Name": _SAMPLE_EMR_NAME,
                "Id": _SAMPLE_EMR_ID,
                "Status": _SAMPLE_EMR_STATUS,
                "LogUri": _SAMPLE_EMR_LOG_URI,
            }
        ]

        std_out = capture_function(self._service.list_emr_clusters, {"no_grid": True}).std_out
        expected_output = "  ".join(
            [_SAMPLE_EMR_NAME, _SAMPLE_EMR_ID, _SAMPLE_EMR_STATE, _SAMPLE_EMR_LOG_URI]
        )
        self.assertEqual(std_out.getvalue().strip().strip(), expected_output)

    def test_list_emr_clusters_only_log_locations(self):
        self._aws_wrapper_mock.get_emr_cluster_details.return_value = [
            {
                "Name": _SAMPLE_EMR_NAME,
                "Id": _SAMPLE_EMR_ID,
                "Status": _SAMPLE_EMR_STATUS,
                "LogUri": _SAMPLE_EMR_LOG_URI,
            },
            {
                "Name": _SAMPLE_EMR_NAME,
                "Id": _SAMPLE_EMR_ID_2,
                "Status": _SAMPLE_EMR_STATUS,
                "LogUri": _SAMPLE_EMR_LOG_URI_2,
            },
        ]

        std_out = capture_function(
            self._service.list_emr_clusters, {"only_log_locations": True}
        ).std_out
        self.assertEqual(
            set(std_out.getvalue().strip().split("\n")),
            {_SAMPLE_EMR_LOG_URI, _SAMPLE_EMR_LOG_URI_2},
        )

    def test_list_emr_clusters_no_clusters(self):
        self._aws_wrapper_mock.get_emr_cluster_details.return_value = []
        self._user_service_mock.active_collector = {"customerAwsAccountId": _SAMPLE_ACCOUNT_ID}

        std_out = capture_function(
            self._service.list_emr_clusters, {"table_format": "plain"}
        ).std_out
        self.assertEqual(std_out.getvalue().strip(), "Name    Id    State    LogUri")

    @patch("montecarlodata.common.resources.click")
    @patch.object(CloudResourceService, "_initialize_events")
    @patch.object(CloudResourceService, "_create_or_update_event_topic")
    @patch.object(CloudResourceService, "_update_event_q")
    @patch.object(CloudResourceService, "_create_topic_subscription")
    @patch.object(CloudResourceService, "_create_event_notification")
    def test_add_events_flow(
        self,
        mock_notification,
        mock_topic_sub,
        mock_q,
        mock_topic,
        mock_setup,
        mock_click,
    ):
        self._user_service_mock.get_collector.return_value = Box(_SAMPLE_COLLECTOR)
        mock_setup.return_value = _SAMPLE_EVENT_DETAILS

        self._service.add_events(**_SAMPLE_EVENT_PAYLOAD)

        self._user_service_mock.get_collector.assert_called_once_with(dc_id=None, agent_id=None)

        mock_setup.assert_called_once_with(
            collector_props=_SAMPLE_COLLECTOR, **_SAMPLE_EVENT_PAYLOAD
        )
        mock_topic.assert_called_once_with(event_details=_SAMPLE_EVENT_DETAILS, skip_prompts=False)
        mock_q.assert_called_once_with(event_details=_SAMPLE_EVENT_DETAILS, skip_prompts=False)
        mock_topic_sub.assert_called_once_with(
            event_details=_SAMPLE_EVENT_DETAILS, skip_prompts=False
        )
        mock_notification.assert_called_once_with(
            event_details=_SAMPLE_EVENT_DETAILS, skip_prompts=False
        )

        mock_click.assert_has_calls(
            [
                call.echo("[Gathering collector and resource details]"),
                call.echo("\nProcess complete. Have a nice day!"),
            ]
        )

    def test_add_events_with_an_inactive_collector(self):
        dc_id = "thor"
        inactive_collector = Box(copy.deepcopy(_SAMPLE_COLLECTOR))
        inactive_collector.active = False

        self._user_service_mock.get_collector.return_value = inactive_collector
        with self.assertRaises(click.exceptions.Abort):
            self._service.add_events(**_SAMPLE_EVENT_PAYLOAD, **{"dc_id": dc_id})
        self._user_service_mock.get_collector.assert_called_once_with(dc_id=dc_id, agent_id=None)

    @patch("montecarlodata.common.resources.AwsClientWrapper")
    @patch.object(CloudResourceService, "_get_relevant_q")
    def test_initialize_events(self, mock_get_q, aws_wrapper):
        derived_bucket_location, dc_outputs, q_arn = (
            "thor",
            {"foo": "bar"},
            "arn:aws:sqs:us-east-1:12345:odin",
        )
        aws_wrapper().get_bucket_location.return_value = derived_bucket_location
        aws_wrapper().region = derived_bucket_location
        aws_wrapper().get_stack_outputs.return_value = dc_outputs
        aws_wrapper().get_caller_identity.return_value = AWSArn(q_arn).account

        mock_get_q.return_value = q_arn

        answer = self._service._initialize_events(
            collector_props=Box(_SAMPLE_COLLECTOR), **_SAMPLE_EVENT_PAYLOAD
        )
        expected = EventProperties(
            event_type=_SAMPLE_EVENT_PAYLOAD["event_type"],
            event_queue_arn=q_arn,
            collection_region=AWSArn(q_arn).region,
            collection_account_id=AWSArn(_SAMPLE_COLLECTOR["stackArn"]).account,
            collection_client=ANY,
            resource_properties=[
                ResourceProperties(
                    account_id=AWSArn(q_arn).account,
                    region=derived_bucket_location,
                    bucket_name=_SAMPLE_EVENT_PAYLOAD["bucket_name"],
                    client=ANY,
                )
            ],
        )
        self.assertEqual(answer, expected)

        aws_wrapper.assert_has_calls(
            [
                call(
                    profile_name=_SAMPLE_EVENT_PAYLOAD["collector_aws_profile"],
                    region_name="us-east-1",
                ),
                call().get_stack_outputs(stack_id=_SAMPLE_COLLECTOR["stackArn"]),
                call(profile_name=None),
                call().get_bucket_location(bucket_name="mcd-test-events"),
                call(
                    profile_name=None,
                    region_name=derived_bucket_location,
                ),
                call().get_caller_identity(),
            ]
        )
        mock_get_q.assert_called_once_with("metadata", dc_outputs)

    @patch("montecarlodata.common.resources.AwsClientWrapper")
    @patch.object(CloudResourceService, "_get_relevant_q")
    @patch.object(CloudResourceService, "parse_buckets_file")
    def test_initialize_events_from_buckets_file(self, mock_parse_buckets, mock_get_q, aws_wrapper):
        derived_bucket_location, dc_outputs, q_arn = (
            "thor",
            {"foo": "bar"},
            "arn:aws:sqs:us-east-1:12345:odin",
        )
        aws_wrapper().get_bucket_location.return_value = derived_bucket_location
        aws_wrapper().region = derived_bucket_location
        aws_wrapper().get_stack_outputs.return_value = dc_outputs
        aws_wrapper().get_caller_identity.return_value = AWSArn(q_arn).account

        mock_get_q.return_value = q_arn
        mock_parse_buckets.return_value = [
            BucketRow(bucket_name="foo"),
            BucketRow(bucket_name="bar"),
            BucketRow(bucket_name="baz"),
        ]

        answer = self._service._initialize_events(
            collector_props=Box(_SAMPLE_COLLECTOR), **_SAMPLE_BUCKETS_FILE_EVENT_PAYLOAD
        )
        expected = EventProperties(
            event_type=_SAMPLE_BUCKETS_FILE_EVENT_PAYLOAD["event_type"],
            event_queue_arn=q_arn,
            collection_region=AWSArn(q_arn).region,
            collection_account_id=AWSArn(_SAMPLE_COLLECTOR["stackArn"]).account,
            collection_client=ANY,
            resource_properties=[
                ResourceProperties(
                    account_id=AWSArn(q_arn).account,
                    region=derived_bucket_location,
                    bucket_name="foo",
                    client=ANY,
                ),
                ResourceProperties(
                    account_id=AWSArn(q_arn).account,
                    region=derived_bucket_location,
                    bucket_name="bar",
                    client=ANY,
                ),
                ResourceProperties(
                    account_id=AWSArn(q_arn).account,
                    region=derived_bucket_location,
                    bucket_name="baz",
                    client=ANY,
                ),
            ],
        )

        self.assertEqual(answer, expected)

        aws_wrapper.assert_has_calls(
            [
                call(
                    profile_name=_SAMPLE_EVENT_PAYLOAD["collector_aws_profile"],
                    region_name="us-east-1",
                ),
                call().get_stack_outputs(stack_id=_SAMPLE_COLLECTOR["stackArn"]),
                call(profile_name=None),
                call().get_bucket_location(bucket_name="foo"),
                call(
                    profile_name=None,
                    region_name=derived_bucket_location,
                ),
                call().get_bucket_location(bucket_name="bar"),
                call().get_bucket_location(bucket_name="baz"),
                call().get_caller_identity(),
                call().get_caller_identity(),
                call().get_caller_identity(),
            ]
        )
        mock_get_q.assert_called_once_with("query-logs", dc_outputs)

    @patch("builtins.open")
    def test_parse_buckets_file(self, mock_open):
        mock_open.return_value = StringIO(_SAMPLE_BUCKETS_FILE)
        answer = self._service.parse_buckets_file("test_filename")
        expected = [
            BucketRow(bucket_name="foo", prefix="", suffix=""),
            BucketRow(bucket_name="bar", prefix="prefix1", suffix=""),
            BucketRow(bucket_name="baz", prefix="", suffix="suffix1"),
            BucketRow(bucket_name="qux", prefix="prefix2", suffix="suffix2"),
        ]

        self.assertEqual(answer, expected)

    def test_get_relevant_q(self):
        m_out, q_out = "loki", "thor"
        sample_outputs = [
            {
                "OutputKey": "MetadataEventQueue",
                "OutputValue": m_out,
                "Description": "Metadata event queue ARN",
            },
            {
                "OutputKey": "PublicIP",
                "OutputValue": "127.0.0.1",
                "Description": "Does stuff",
            },
            {
                "OutputKey": "QueryLogEventQueue",
                "OutputValue": q_out,
                "Description": "Query log event queue ARN",
            },
        ]

        self.assertEqual(
            self._service._get_relevant_q(event_type="metadata", outputs=BoxList(sample_outputs)),
            m_out,
        )
        self.assertEqual(
            self._service._get_relevant_q(event_type="query-logs", outputs=BoxList(sample_outputs)),
            q_out,
        )
        with self.assertRaises(click.exceptions.Abort):
            self._service._get_relevant_q(
                event_type="invalid", outputs=BoxList(sample_outputs)
            )  # bad event type
        with self.assertRaises(click.exceptions.Abort):
            self._service._get_relevant_q(
                event_type="metadata", outputs=Box({})
            )  # missing event config

    @patch("montecarlodata.common.resources.prompt_connection")
    @patch.object(CloudResourceService, "_extend_topic_policy")
    @patch.object(CloudResourceService, "_is_preexisting_topic_policy")
    @patch.object(CloudResourceService, "_get_preexisting_topic_policy")
    @patch.object(CloudResourceService, "_save_event_properties")
    def test_create_or_update_event_topic(
        self,
        mock_save_event_properties,
        mock_get_preexisting,
        mock_is_preexisting,
        mock_extend_policy,
        mock_prompt,
    ):
        attributes = Box({"Condition": {"StringEquals": {"aws:SourceArn": "foo"}}})

        events = copy.deepcopy(_SAMPLE_EVENT_DETAILS)
        for resource in events.resource_properties:
            region = resource.region
            topic_arn = (
                f"arn:aws:sns:{region}:12345:monte-carlo-data-{region}-metadata-events-topic"
            )
            resource.client.create_sns_topic.return_value = topic_arn
            resource.client.get_topic_attributes.return_value = attributes

        mock_get_preexisting.return_value = attributes
        mock_is_preexisting.return_value = False
        mock_extend_policy.return_value = attributes

        self._service._create_or_update_event_topic(event_details=events, skip_prompts=True)

        mock_is_preexisting.assert_has_calls(
            [
                call(
                    topic_policy_id="monte-carlo-data-metadata-loki-MetadataEventQueue-42-ID",
                    topic_attributes=attributes,
                ),
                call(
                    topic_policy_id="monte-carlo-data-metadata-loki-MetadataEventQueue-42-ID",
                    topic_attributes=attributes,
                ),
            ]
        )
        mock_extend_policy.assert_has_calls(
            [
                call(
                    topic_attributes=Box({"Condition": {"StringEquals": {"aws:SourceArn": "foo"}}}),
                    new_statements=[
                        {
                            "Sid": "monte-carlo-data-metadata-loki-MetadataEventQueue-42-ID",
                            "Effect": "Allow",
                            "Principal": {"AWS": "123456789"},
                            "Action": "sns:Subscribe",
                            "Resource": "arn:aws:sns:us-east-1:12345:monte-carlo-data-us-east-1-metadata-events-topic",
                        }
                    ],
                    update_statements={
                        "monte-carlo-data-metadata-monte-carlo-data-us-east-1-metadata-events-topic-ID": Box(
                            {
                                "Condition": {
                                    "StringEquals": {
                                        "aws:SourceArn": [
                                            "foo",
                                            "arn:aws:s3:::mcd-test-events",
                                        ]
                                    }
                                }
                            }
                        )
                    },
                ),
                call(
                    topic_attributes=Box({"Condition": {"StringEquals": {"aws:SourceArn": "foo"}}}),
                    new_statements=[
                        {
                            "Sid": "monte-carlo-data-metadata-loki-MetadataEventQueue-42-ID",
                            "Effect": "Allow",
                            "Principal": {"AWS": "123456789"},
                            "Action": "sns:Subscribe",
                            "Resource": "arn:aws:sns:us-east-2:12345:monte-carlo-data-us-east-2-metadata-events-topic",
                        }
                    ],
                    update_statements={
                        "monte-carlo-data-metadata-monte-carlo-data-us-east-2-metadata-events-topic-ID": Box(
                            {
                                "Condition": {
                                    "StringEquals": {
                                        "aws:SourceArn": [
                                            "foo",
                                            "arn:aws:s3:::mcd-test-events-us-east-2",
                                        ]
                                    }
                                }
                            }
                        )
                    },
                ),
            ]
        )

        for resource in events.resource_properties:
            resource.client.set_topic_attributes.assert_called_once_with(
                arn=resource.topic_arn, name="Policy", value=attributes
            )
            resource.client.get_topic_attributes.assert_called_once_with(arn=resource.topic_arn)
            resource.client.create_sns_topic.assert_called_once_with(name=resource.topic_name)
        mock_prompt.assert_has_calls(
            [
                call(
                    message="Create topic 'monte-carlo-data-us-east-1-metadata-events-topic' in 'us-east-1' (123456789)? "
                    "This is an idempotent operation.",
                    skip_prompt=True,
                ),
                call(message="Please confirm", skip_prompt=True),
                call(
                    message="Create topic 'monte-carlo-data-us-east-2-metadata-events-topic' in 'us-east-2' (123456789)? "
                    "This is an idempotent operation.",
                    skip_prompt=True,
                ),
                call(message="Please confirm", skip_prompt=True),
            ]
        )

        mock_save_event_properties.assert_called()

    def test_create_or_update_event_topic_arn_invalid(self):
        events = copy.deepcopy(_SAMPLE_EVENT_DETAILS)

        with self.assertRaises(click.exceptions.Abort):
            for resource in events.resource_properties:
                resource.topic_arn = "foo"
            self._service._create_or_update_event_topic(
                event_details=events, skip_prompts=True
            )  # invalid ARN

    def test_create_or_update_event_topic_region_invalid(self):
        events = copy.deepcopy(_SAMPLE_EVENT_DETAILS)

        with self.assertRaises(click.exceptions.Abort):
            for resource in events.resource_properties:
                resource.topic_arn = (
                    "arn:aws:sns:us-west-1:12345:monte-carlo-data-us-east-1-metadata-events-topic"
                )
            self._service._create_or_update_event_topic(
                event_details=events, skip_prompts=True
            )  # region mismatch

    def test_preexisting_topic_policy(self):
        sid, topic_attributes = "foo", Box({"Attributes": {"Policy": '{"Statement": []}'}})
        self.assertFalse(self._service._is_preexisting_topic_policy(sid, topic_attributes))

        topic_attributes = Box({"Attributes": {"Policy": '{"Statement": [{"Sid": "foo"}]}'}})
        self.assertTrue(self._service._is_preexisting_topic_policy(sid, topic_attributes))

    def test_extend_topic_policy(self):
        topic_attributes = Box({"Attributes": {"Policy": '{"Statement": [{"Sid": "foo"}]}'}})
        new_statements = [{"x": "y"}]
        self.assertEqual(
            self._service._extend_topic_policy(
                topic_attributes, new_statements, update_statements=[]
            ),
            '{"Statement": [{"Sid": "foo"}, {"x": "y"}]}',
        )

    def test_extend_topic_policy_with_updates(self):
        topic_attributes = Box({"Attributes": {"Policy": '{"Statement": [{"Sid": "foo"}]}'}})
        new_statements = [{"x": "y"}]
        update_statements = {"foo": {"Sid": "foo", "bar": "baz"}}
        self.assertEqual(
            self._service._extend_topic_policy(
                topic_attributes, new_statements, update_statements=update_statements
            ),
            '{"Statement": [{"Sid": "foo", "bar": "baz"}, {"x": "y"}]}',
        )

    def test_extend_topic_policy_topic_attributes_without_sid(self):
        topic_attributes = Box({"Attributes": {"Policy": '{"Statement": [{"foo": "bar"}]}'}})
        new_statements = [{"x": "y"}]
        update_statements = {"foo": {"Sid": "foo", "bar": "baz"}}
        self.assertEqual(
            '{"Statement": [{"foo": "bar"}, {"x": "y"}]}',
            self._service._extend_topic_policy(topic_attributes, new_statements, update_statements),
        )

    @patch("montecarlodata.common.resources.prompt_connection")
    @patch.object(CloudResourceService, "_save_event_properties")
    def test_update_event_q(self, mock_save_event_properties, mock_prompt):
        name = "loki-MetadataEventQueue-42"
        _SAMPLE_DC_CLIENT.get_q_attributes.return_value = None

        events = copy.deepcopy(_SAMPLE_EVENT_DETAILS)
        for resource in events.resource_properties:
            resource.topic_arn = f"arn:aws:sns:{resource.region}:12345:monte-carlo-data-{resource.region}-metadata-events-topic"

        self._service._update_event_q(event_details=events, skip_prompts=True)

        events.collection_client.set_q_attributes.assert_called_once_with(
            name=name,
            attributes={
                "Policy": '{"Version": "2008-10-17", "Statement": [{"Sid": "__owner", "Effect": "Allow", "Principal": '
                '{"AWS": "arn:aws:iam::123456789:root"}, "Action": "SQS:*", "Resource": '
                '"arn:aws:sqs:us-east-1:123456789:loki-MetadataEventQueue-42"}, '
                '{"Sid": "__sender", "Effect": "Allow", "Principal": {"AWS": "*"}, "Action": '
                '"SQS:SendMessage", "Resource": "arn:aws:sqs:us-east-1:123456789:loki-MetadataEventQueue-42",'
                ' "Condition": {"ArnLike": {"aws:SourceArn": '
                '["arn:aws:sns:us-east-1:12345:monte-carlo-data-us-east-1-metadata-events-topic",'
                ' "arn:aws:sns:us-east-2:12345:monte-carlo-data-us-east-2-metadata-events-topic"]}}}]}'
            },
        )
        events.collection_client.get_q_attributes.assert_called_once_with(
            name=name, attributes=["Policy"]
        )
        mock_prompt.assert_called_once_with(message="Please confirm", skip_prompt=True)
        mock_save_event_properties.assert_called()

    @patch("montecarlodata.common.resources.prompt_connection")
    @patch.object(CloudResourceService, "_extend_q_policy")
    @patch.object(CloudResourceService, "_save_event_properties")
    def test_update_event_q_with_existing(
        self, mock_save_event_properties, mock_extend, mock_prompt
    ):
        name, mock_extend.return_value = "loki-MetadataEventQueue-42", {"foo": "bar"}

        _SAMPLE_DC_CLIENT.get_q_attributes.return_value = Box({"Attributes": {"Policy": "{}"}})
        self._service._update_event_q(event_details=_SAMPLE_EVENT_DETAILS, skip_prompts=True)

        _SAMPLE_DC_CLIENT.get_q_attributes.assert_called_once_with(name=name, attributes=["Policy"])
        _SAMPLE_DC_CLIENT.set_q_attributes.assert_called_once_with(
            name=name, attributes={"Policy": '{"foo": "bar"}'}
        )
        mock_prompt.assert_called_once_with(message="Please confirm", skip_prompt=True)
        mock_save_event_properties.assert_called()

    @patch("montecarlodata.common.resources.prompt_connection")
    @patch.object(CloudResourceService, "_extend_q_policy")
    def test_update_event_q_with_no_delta(self, mock_extend, mock_prompt):
        name, mock_extend.return_value = "loki-MetadataEventQueue-42", {"foo": "bar"}

        _SAMPLE_DC_CLIENT.get_q_attributes.return_value = Box(
            {"Attributes": {"Policy": '{"foo": "bar"}'}}
        )
        self._service._update_event_q(event_details=_SAMPLE_EVENT_DETAILS, skip_prompts=True)

        _SAMPLE_DC_CLIENT.get_q_attributes.assert_called_once_with(name=name, attributes=["Policy"])
        _SAMPLE_DC_CLIENT.set_q_attributes.assert_not_called()
        mock_prompt.assert_not_called()

    def test_extend_q_policy(self):
        events = copy.deepcopy(_SAMPLE_EVENT_DETAILS)
        for resource in events.resource_properties:
            resource.topic_arn = "bar"

        # Existing is string
        existing_policy = Box(
            {
                "Statement": [
                    {
                        "Sid": "__sender",
                        "Condition": {"ArnLike": {"aws:SourceArn": "foo"}},
                    }
                ]
            }
        )
        self.assertEqual(
            self._service._extend_q_policy(
                existing_policy=existing_policy,
                event_details=events,
                topic_arns=["bar"],
            ),
            {
                "Statement": [
                    {
                        "Sid": "__sender",
                        "Condition": {"ArnLike": {"aws:SourceArn": ["foo", "bar"]}},
                    }
                ]
            },
        )

        # Existing is list
        existing_policy = Box(
            {
                "Statement": [
                    {
                        "Sid": "__sender",
                        "Condition": {"ArnLike": {"aws:SourceArn": ["foo", "qux"]}},
                    }
                ]
            }
        )
        self.assertEqual(
            self._service._extend_q_policy(
                existing_policy=existing_policy,
                event_details=events,
                topic_arns=["bar"],
            ),
            {
                "Statement": [
                    {
                        "Sid": "__sender",
                        "Condition": {"ArnLike": {"aws:SourceArn": ["foo", "qux", "bar"]}},
                    }
                ]
            },
        )

        # Existing has duplicate
        existing_policy = Box(
            {
                "Statement": [
                    {
                        "Sid": "__sender",
                        "Condition": {"ArnLike": {"aws:SourceArn": "bar"}},
                    }
                ]
            }
        )
        self.assertEqual(
            self._service._extend_q_policy(
                existing_policy=existing_policy,
                event_details=events,
                topic_arns=["bar"],
            ),
            {
                "Statement": [
                    {
                        "Sid": "__sender",
                        "Condition": {"ArnLike": {"aws:SourceArn": "bar"}},
                    }
                ]
            },
        )
        existing_policy = Box(
            {
                "Statement": [
                    {
                        "Sid": "__sender",
                        "Condition": {"ArnLike": {"aws:SourceArn": ["foo", "bar"]}},
                    }
                ]
            }
        )
        self.assertEqual(
            self._service._extend_q_policy(
                existing_policy=existing_policy,
                event_details=events,
                topic_arns=["foo", "bar"],
            ),
            {
                "Statement": [
                    {
                        "Sid": "__sender",
                        "Condition": {"ArnLike": {"aws:SourceArn": ["foo", "bar"]}},
                    }
                ]
            },
        )

        # Incompatible format
        with self.assertRaises(click.exceptions.Abort):
            self._service._extend_q_policy(
                existing_policy=Box({"Statement": [{"Sid": "unk"}]}),
                event_details=events,
                topic_arns=["bar"],
            )

    @patch("montecarlodata.common.resources.prompt_connection")
    @patch.object(CloudResourceService, "_merge_event_notifications")
    @patch.object(CloudResourceService, "_save_event_properties")
    def test_create_event_notification(
        self, mock_save_event_properties, mock_merge_events, mock_prompt
    ):
        events = copy.deepcopy(_SAMPLE_EVENT_DETAILS)
        new_config, existing_config = {"foo": "bar"}, {"bar": "foo"}

        for resource in events.resource_properties:
            resource.topic_arn = (
                "arn:aws:sns:us-east-1:12345:monte-carlo-data-us-east-1-metadata-events-topic"
            )
            resource.prefix = "lake"
            resource.client.get_bucket_event_config.return_value = existing_config

        mock_merge_events.return_value = new_config
        self._service._create_event_notification(event_details=events, skip_prompts=True)

        expected_merge_events_calls = [
            call(
                setting_id="monte-carlo-data-us-east-1-metadata-events-topic-lake-notification",
                existing_config=existing_config,
                resource_details=resource,
            )
            for resource in events.resource_properties
        ]
        self.assertEqual(mock_merge_events.call_args_list, expected_merge_events_calls)

        for resource in events.resource_properties:
            resource.client.get_bucket_event_config.assert_called_once_with(
                name=resource.bucket_name
            )
            resource.client.set_bucket_event_config.assert_called_once_with(
                name=resource.bucket_name, notification_config=new_config
            )

        expected_prompt_calls = [
            call(message="Please confirm", skip_prompt=True) for _ in events.resource_properties
        ]
        self.assertEqual(mock_prompt.call_args_list, expected_prompt_calls)

        mock_save_event_properties.assert_called()

    @patch("montecarlodata.common.resources.prompt_connection")
    @patch.object(CloudResourceService, "_merge_event_notifications")
    @patch.object(CloudResourceService, "_save_event_properties")
    def test_create_event_notification_with_no_delta(
        self, mock_save_event_properties, mock_merge_events, mock_prompt
    ):
        events = copy.deepcopy(_SAMPLE_EVENT_DETAILS)
        new_config, existing_config = {"foo": "bar"}, {"foo": "bar"}

        for resource in events.resource_properties:
            resource.topic_arn = (
                "arn:aws:sns:us-east-1:12345:monte-carlo-data-us-east-1-metadata-events-topic"
            )
            resource.client.get_bucket_event_config.return_value = existing_config

        mock_merge_events.return_value = new_config
        self._service._create_event_notification(event_details=events, skip_prompts=True)
        expected_merge_events_calls = [
            call(
                setting_id="monte-carlo-data-us-east-1-metadata-events-topic-notification",
                existing_config=existing_config,
                resource_details=resource,
            )
            for resource in events.resource_properties
        ]
        self.assertEqual(mock_merge_events.call_args_list, expected_merge_events_calls)

        for resource in events.resource_properties:
            resource.client.get_bucket_event_config.assert_called_once_with(
                name=resource.bucket_name
            )
            resource.client.set_bucket_event_config.assert_not_called()

        mock_prompt.assert_not_called()
        mock_save_event_properties.assert_called()

    def test_merge_event_notifications(self):
        setting_id = "foo"
        events = copy.deepcopy(_SAMPLE_EVENT_DETAILS)

        for resource in events.resource_properties:
            resource.topic_arn = f"arn:aws:sns:{resource.region}:12345:monte-carlo-data-{resource.region}-metadata-events-topic"
            expected_config = {
                "Id": setting_id,
                "TopicArn": resource.topic_arn,
                "Events": ["s3:ObjectCreated:*", "s3:ObjectRemoved:*"],
                "Filter": {
                    "Key": {
                        "FilterRules": [
                            {"Name": "Prefix", "Value": ""},
                            {"Name": "Suffix", "Value": ""},
                        ]
                    }
                },
            }
            # ID already exists
            existing_setting = {"TopicConfigurations": [{"Id": setting_id}]}
            self.assertEqual(
                self._service._merge_event_notifications(setting_id, existing_setting, resource),
                existing_setting,
            )

            # Topic already exits
            existing_setting = {
                "TopicConfigurations": [{"Id": "bar"}],
                "QueueConfigurations": [{"foo": "bar"}],
            }
            self.assertEqual(
                self._service._merge_event_notifications(setting_id, existing_setting, resource),
                {
                    "TopicConfigurations": [{"Id": "bar"}, expected_config],
                    "QueueConfigurations": [{"foo": "bar"}],
                },
            )

            # No topic config
            self.assertEqual(
                self._service._merge_event_notifications(setting_id, {}, resource),
                {"TopicConfigurations": [expected_config]},
            )

            existing_setting = {"QueueConfigurations": [{"foo": "bar"}]}
            self.assertEqual(
                self._service._merge_event_notifications(setting_id, existing_setting, resource),
                {
                    "TopicConfigurations": [expected_config],
                    "QueueConfigurations": [{"foo": "bar"}],
                },
            )

    @patch("montecarlodata.common.resources.prompt_connection")
    @patch.object(CloudResourceService, "_save_event_properties")
    def test_create_topic_subscription(self, mock_save_event_properties, mock_prompt):
        events = copy.deepcopy(_SAMPLE_EVENT_DETAILS)
        for resource in events.resource_properties:
            resource.client.list_topic_subscriptions.return_value = None
        self._service._create_topic_subscription(event_details=events, skip_prompts=True)

        for resource in events.resource_properties:
            resource.client.list_topic_subscriptions.assert_called_once_with(arn=resource.topic_arn)
            resource.client.subscribe_to_topic.assert_called_once_with(
                arn=resource.topic_arn,
                endpoint=events.event_queue_arn,
                attributes={"RawMessageDelivery": "true"},
            )

        mock_prompt.assert_has_calls(
            [
                call(message="Please confirm", skip_prompt=True),
                call(message="Please confirm", skip_prompt=True),
            ]
        )
        mock_save_event_properties.assert_called()

    @patch("montecarlodata.common.resources.prompt_connection")
    @patch.object(CloudResourceService, "_save_event_properties")
    def test_create_topic_subscription_with_preexisting_endpoint(
        self, mock_save_event_properties, mock_prompt
    ):
        events = copy.deepcopy(_SAMPLE_EVENT_DETAILS)
        for resource in events.resource_properties:
            resource.client.list_topic_subscriptions.return_value = [
                [
                    {
                        "SubscriptionArn": "odin",
                        "Owner": "12345",
                        "Protocol": "sqs",
                        "Endpoint": events.event_queue_arn,
                        "TopicArn": resource.topic_arn,
                    }
                ]
            ]

        self._service._create_topic_subscription(event_details=events, skip_prompts=True)

        for resource in events.resource_properties:
            resource.client.list_topic_subscriptions.assert_called_once_with(arn=resource.topic_arn)
            _SAMPLE_CUST_RESOURCES_CLIENT.subscribe_to_topic.cassert_not_called()

        mock_prompt.assert_not_called()
        mock_save_event_properties.assert_called()

    @patch("montecarlodata.common.resources.AwsClientWrapper")
    def test_get_dc_resource_props(self, aws_wrapper_mock):
        arn = "arn:aws:cloudformation:us-east-1:1234:stack/foo/bar"
        collector_props = Box({"stackArn": arn})

        self.maxDiff = None
        self.assertEqual(
            self._service.get_dc_resource_props(collector_props=collector_props),
            DcResourceProperties(
                collector_arn=AWSArn(arn),
                collector_props=collector_props,
                collection_region="us-east-1",
                resources_region="us-east-1",
                collection_client=aws_wrapper_mock(),
                resources_client=aws_wrapper_mock(),
            ),
        )
        aws_wrapper_mock.assert_has_calls(
            [
                call(
                    profile_name=None,
                    region_name="us-east-1",
                ),
                call(
                    profile_name=None,
                    region_name="us-east-1",
                ),
            ]
        )

    def test_parse_stack_prop_list(self):
        self.assertEqual(
            self._service._parse_stack_prop_list(
                key="foo", val="bar", struct=[{"foo": "baz", "bar": "qux"}]
            ),
            Box({"baz": "qux"}),
        )



import base64
from unittest import TestCase
from unittest.mock import Mock, patch

from montecarlodata.common.user import UserService
from montecarlodata.integrations.onboarding.warehouse.warehouses import (
    WarehouseOnboardingService,
)
from montecarlodata.queries.onboarding import (
    TEST_BQ_CRED_MUTATION,
    TEST_DATABASE_CRED_MUTATION,
    TEST_SNOWFLAKE_CRED_MUTATION,
)
from montecarlodata.utils import AwsClientWrapper, GqlWrapper
from tests.test_base_onboarding import _SAMPLE_BASE_OPTIONS
from tests.test_common_user import _SAMPLE_CONFIG


class WarehouseOnBoardingTest(TestCase):
    def setUp(self) -> None:
        self._user_service_mock = Mock(autospec=UserService)
        self._request_wrapper_mock = Mock(autospec=GqlWrapper)
        self._aws_wrapper_mock = Mock(autospec=AwsClientWrapper)

        self._service = WarehouseOnboardingService(
            _SAMPLE_CONFIG,
            command_name="test",
            request_wrapper=self._request_wrapper_mock,
            aws_wrapper=self._aws_wrapper_mock,
            user_service=self._user_service_mock,
        )

    @patch.object(WarehouseOnboardingService, "onboard")
    def test_redshift_flow(self, onboard_mock):
        expected_options = {
            **{"connectionType": "redshift", "warehouseType": "redshift"},
            **_SAMPLE_BASE_OPTIONS,
        }

        self._service.onboard_redshift(**_SAMPLE_BASE_OPTIONS)
        onboard_mock.assert_called_once_with(
            validation_query=TEST_DATABASE_CRED_MUTATION,
            validation_response="testDatabaseCredentials",
            connection_type="redshift",
            **expected_options,
        )

    @patch.object(WarehouseOnboardingService, "onboard")
    def test_snowflake_flow(self, onboard_mock):
        expected_options = {**{"warehouseType": "snowflake"}, **_SAMPLE_BASE_OPTIONS}

        self._service.onboard_snowflake(**_SAMPLE_BASE_OPTIONS)
        onboard_mock.assert_called_once_with(
            validation_query=TEST_SNOWFLAKE_CRED_MUTATION,
            validation_response="testSnowflakeCredentials",
            connection_type="snowflake",
            **expected_options,
        )

    @patch.object(WarehouseOnboardingService, "onboard")
    @patch("montecarlodata.integrations.onboarding.warehouse.warehouses.read_as_base64")
    def test_snowflake_flow_with_private_key(self, read_as_base64_mock, onboard_mock):
        file_path = "/tmp/my_private_key"
        private_key = "private_key"
        base64_private_key = base64.b64encode(private_key.encode("utf-8"))
        input_options = {"private_key": file_path, **_SAMPLE_BASE_OPTIONS}

        expected_options = {
            **{
                "warehouseType": "snowflake",
                "private_key": base64_private_key.decode(),
            },
            **_SAMPLE_BASE_OPTIONS,
        }

        read_as_base64_mock.return_value = base64_private_key

        self._service.onboard_snowflake(**input_options)
        read_as_base64_mock.assert_called_once_with(file_path)
        onboard_mock.assert_called_once_with(
            validation_query=TEST_SNOWFLAKE_CRED_MUTATION,
            validation_response="testSnowflakeCredentials",
            connection_type="snowflake",
            **expected_options,
        )

    @patch.object(WarehouseOnboardingService, "onboard")
    @patch("montecarlodata.integrations.onboarding.warehouse.warehouses.read_as_base64")
    def test_snowflake_flow_with_private_key_and_passphrase(
        self, read_as_base64_mock, onboard_mock
    ):
        file_path = "/tmp/my_private_key"
        private_key = "private_key"
        passphrase = "foobar123"
        base64_private_key = base64.b64encode(private_key.encode("utf-8"))
        input_options = {
            "private_key": file_path,
            "private_key_passphrase": passphrase,
            **_SAMPLE_BASE_OPTIONS,
        }

        expected_options = {
            **{
                "warehouseType": "snowflake",
                "private_key": base64_private_key.decode(),
                "private_key_passphrase": passphrase,
            },
            **_SAMPLE_BASE_OPTIONS,
        }

        read_as_base64_mock.return_value = base64_private_key

        self._service.onboard_snowflake(**input_options)
        read_as_base64_mock.assert_called_once_with(file_path)
        onboard_mock.assert_called_once_with(
            validation_query=TEST_SNOWFLAKE_CRED_MUTATION,
            validation_response="testSnowflakeCredentials",
            connection_type="snowflake",
            **expected_options,
        )

    @patch.object(WarehouseOnboardingService, "onboard")
    @patch("montecarlodata.integrations.onboarding.warehouse.warehouses.read_as_base64")
    def test_bq_flow(self, read_as_base64_mock, onboard_mock):
        file_path, service_json = "foo", "bar"
        base64_service_json = base64.b64encode(service_json.encode("utf-8"))

        input_options = {"ServiceFile": file_path, **_SAMPLE_BASE_OPTIONS}
        expected_options = {
            **{
                "warehouseType": "bigquery",
                "serviceJson": base64_service_json.decode(),
            },
            **_SAMPLE_BASE_OPTIONS,
        }

        read_as_base64_mock.return_value = base64_service_json

        self._service.onboard_bq(**input_options)
        read_as_base64_mock.assert_called_once_with(file_path)
        onboard_mock.assert_called_once_with(
            validation_query=TEST_BQ_CRED_MUTATION,
            validation_response="testBqCredentials",
            connection_type="bigquery",
            **expected_options,
        )



from typing import Dict, Optional
from uuid import UUID

import click

from montecarlodata.common.common import create_mc_client
from montecarlodata.dataimport.dbt import DbtImportService


@click.group(help="Import data.", name="import")
def import_subcommand():
    """
    Group for any import related subcommands
    """
    pass


@import_subcommand.command()
@click.option(
    "--project-name",
    type=click.STRING,
    default="default-project",
    show_default=True,
    help="Project name (perhaps a logical group of dbt models,"
    " analogous to a project in dbt Cloud)",
)
@click.option(
    "--job-name",
    type=click.STRING,
    default="default-job",
    show_default=True,
    help="Job name (perhaps a logical sequence of dbt executions, analogous to a job in dbt Cloud)",
)
@click.option(
    "--manifest",
    type=click.Path(exists=True),
    required=True,
    help="Path to the dbt manifest file (manifest.json)",
)
@click.option(
    "--run-results",
    type=click.Path(exists=True),
    required=True,
    help="Path to the dbt run results file (run_results.json)",
)
@click.option(
    "--logs",
    type=click.Path(exists=True),
    required=False,
    help="Path to a file containing dbt run logs",
)
@click.option(
    "--connection-id",
    type=click.UUID,
    required=False,
    help="Identifier of warehouse or lake connection to use to resolve dbt models to tables. "
    "Required if you have more than one warehouse or lake connection.",
)
@click.pass_obj
def dbt_run(
    ctx: Dict,
    project_name: str,
    job_name: str,
    manifest: str,
    run_results: str,
    logs: Optional[str],
    connection_id: Optional[UUID],
):
    """
    Import dbt run artifacts.
    """
    DbtImportService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="import dbt_run",
    ).import_run(
        project_name=project_name,
        job_name=job_name,
        manifest_path=manifest,
        run_results_path=run_results,
        logs_path=logs,
        connection_id=connection_id,
    )



from unittest import TestCase
from unittest.mock import Mock, patch

import click

from montecarlodata.common.user import UserService
from montecarlodata.integrations.onboarding.fields import (
    EXPECTED_ADD_BI_RESPONSE_FIELD,
    LOOKER_BI_TYPE,
)
from montecarlodata.integrations.onboarding.self_hosted_credentials import (
    SelfHostedCredentialOnboardingService,
)
from montecarlodata.queries.onboarding import (
    ADD_BI_CONNECTION_MUTATION,
    TEST_SELF_HOSTED_CRED_MUTATION,
)
from montecarlodata.utils import AwsClientWrapper, GqlWrapper
from tests.test_base_onboarding import _SAMPLE_BASE_OPTIONS
from tests.test_common_user import _SAMPLE_CONFIG


class SelfHostedCredentialOnboardingTest(TestCase):
    def setUp(self) -> None:
        self._user_service_mock = Mock(autospec=UserService)
        self._request_wrapper_mock = Mock(autospec=GqlWrapper)
        self._aws_wrapper_mock = Mock(autospec=AwsClientWrapper)

        self._service = SelfHostedCredentialOnboardingService(
            _SAMPLE_CONFIG,
            command_name="test",
            request_wrapper=self._request_wrapper_mock,
            aws_wrapper=self._aws_wrapper_mock,
            user_service=self._user_service_mock,
        )

    @patch.object(SelfHostedCredentialOnboardingService, "onboard")
    def test_onboard_warehouse(self, onboard_mock):
        key = "arn:aws:secretsmanager:us-east-1:test_account:secret:test-key"
        conn_type = "snowflake"
        expected_options = {
            "connectionType": conn_type,
            "region": "us-east-1",
            "warehouseType": "snowflake",
            **_SAMPLE_BASE_OPTIONS,
        }

        self._service.onboard_connection(conn_type, key, **_SAMPLE_BASE_OPTIONS)
        onboard_mock.assert_called_once_with(
            validation_query=TEST_SELF_HOSTED_CRED_MUTATION,
            validation_response="testSelfHostedCredentials",
            connection_type=conn_type,
            self_hosting_key=key,
            **expected_options,
        )

    @patch.object(SelfHostedCredentialOnboardingService, "onboard")
    def test_onboard_lake(self, onboard_mock):
        key = "arn:aws:secretsmanager:us-east-1:test_account:secret:test-key"
        conn_type = "presto"
        expected_options = {
            "connectionType": conn_type,
            "region": "us-east-1",
            **_SAMPLE_BASE_OPTIONS,
        }

        self._service.onboard_connection(conn_type, key, **_SAMPLE_BASE_OPTIONS)
        onboard_mock.assert_called_once_with(
            validation_query=TEST_SELF_HOSTED_CRED_MUTATION,
            validation_response="testSelfHostedCredentials",
            connection_type=conn_type,
            self_hosting_key=key,
            **expected_options,
        )

    @patch.object(SelfHostedCredentialOnboardingService, "onboard")
    def test_onboard_bi(self, onboard_mock):
        key = "arn:aws:secretsmanager:us-east-1:test_account:secret:test-key"
        conn_type = "looker"
        expected_options = {
            "connectionType": conn_type,
            "region": "us-east-1",
            **_SAMPLE_BASE_OPTIONS,
        }

        self._service.onboard_connection(conn_type, key, **_SAMPLE_BASE_OPTIONS)
        onboard_mock.assert_called_once_with(
            validation_query=TEST_SELF_HOSTED_CRED_MUTATION,
            validation_response="testSelfHostedCredentials",
            connection_type=conn_type,
            connection_query=ADD_BI_CONNECTION_MUTATION,
            connection_response=EXPECTED_ADD_BI_RESPONSE_FIELD,
            warehouse_type=LOOKER_BI_TYPE,
            self_hosting_key=key,
            **expected_options,
        )

    def test_bad_arn(self):
        with self.assertRaises(click.Abort):
            self._service.onboard_connection("presto", "invalid_arn")
